using System;
using Server.Items;

namespace Server
{
	public class GenTeleporter
	{
		public GenTeleporter()
		{
		}

		public static void Initialize()
		{
			Server.Commands.Register( "TelGen", AccessLevel.Administrator, new CommandEventHandler( GenTeleporter_OnCommand ) );
		}

		private static void GenTeleporter_OnCommand( CommandEventArgs e )
		{
			World.Broadcast( 0x35, true, "Teleporters is being generated, please wait." );

			DateTime startTime = DateTime.Now;

			int count = new TeleportersCreator().CreateTeleporters();

			DateTime endTime = DateTime.Now;

			World.Broadcast( 0x35, true, "{0} teleporters has been created. The entire process took {1:F1} seconds.", count, (endTime - startTime).TotalSeconds );
		}

		public class TeleportersCreator
		{
			private int m_Count;
			
			public TeleportersCreator()
			{
			}

			public void CreateTeleporter( Point3D pointLocation, Point3D pointDestination, Map mapLocation, Map mapDestination, bool back )
			{
				m_Count++;
				
				Teleporter tel = new Teleporter( pointDestination, mapDestination );

				tel.MoveToWorld( pointLocation, mapLocation );

				if ( back )
				{
					m_Count++;

					Teleporter telBack = new Teleporter( pointLocation, mapLocation );

					telBack.MoveToWorld( pointDestination, mapDestination );
				}
			}

			public void CreateTeleporter( int xLoc, int yLoc, int zLoc, int xDest, int yDest, int zDest, Map map, bool back )
			{
				CreateTeleporter( new Point3D( xLoc, yLoc, zLoc ), new Point3D( xDest, yDest, zDest ), map, map, back);
			}

			public void CreateTeleportersMap( Map map )
			{
				// Shame
				CreateTeleporter( 512, 1559, 0, 5394, 127, 0, map, true );
				CreateTeleporter( 513, 1559, 0, 5395, 127, 0, map, true );
				CreateTeleporter( 514, 1559, 0, 5396, 127, 0, map, true );
				CreateTeleporter( 5490, 19, -25, 5514, 10, 5, map, true );
				CreateTeleporter( 5875, 19, -5, 5507, 162, 5, map, true );
				CreateTeleporter( 5604, 102, 5, 5514, 147, 25, map, true );
				
				// Hythloth
				CreateTeleporter( 4721, 3813, 4, 5904, 16, 61, map, true );
				CreateTeleporter( 4722, 3813, 4, 5905, 16, 61, map, true );
				CreateTeleporter( 4723, 3813, 4, 5906, 16, 61, map, true );
				CreateTeleporter( 6040, 192, 12, 6059, 88, 24, map, true );
				CreateTeleporter( 6040, 193, 12, 6059, 89, 24, map, true );
				CreateTeleporter( 6040, 194, 12, 6059, 90, 24, map, true );
				CreateTeleporter( 5920, 168, 22, 6083, 144, -20, map, true );
				CreateTeleporter( 5920, 169, 22, 6083, 145, -20, map, true );
				CreateTeleporter( 5920, 170, 22, 6083, 146, -20, map, true );
				CreateTeleporter( 5972, 168, 0, 5906, 96, 0, map, true );

				// Covetous
				CreateTeleporter( 2498, 916, 0, 5455, 1864, 0, map, true );
				CreateTeleporter( 2499, 916, 0, 5456, 1864, 0, map, true );
				CreateTeleporter( 2500, 916, 0, 5457, 1864, 0, map, true );
				CreateTeleporter( 2384, 836, 0, 5615, 1996, 0, map, true );
				CreateTeleporter( 2384, 837, 0, 5615, 1997, 0, map, true );
				CreateTeleporter( 2384, 838, 0, 5615, 1998, 0, map, true );
				CreateTeleporter( 2420, 883, 0, 5392, 1959, 0, map, true );
				CreateTeleporter( 2421, 883, 0, 5393, 1959, 0, map, true );
				CreateTeleporter( 2422, 883, 0, 5394, 1959, 0, map, true );
				CreateTeleporter( 2455, 858, 0, 5388, 2027, 0, map, true );
				CreateTeleporter( 2456, 858, 0, 5389, 2027, 0, map, true );
				CreateTeleporter( 2457, 858, 0, 5390, 2027, 0, map, true );
				CreateTeleporter( 2544, 850, 0, 5578, 1927, 0, map, true );
				CreateTeleporter( 2545, 850, 0, 5579, 1927, 0, map, true );
				CreateTeleporter( 2546, 850, 0, 5580, 1927, 0, map, true );
				CreateTeleporter( 5468, 1804, 0, 5592, 1840, 0, map, false );
				CreateTeleporter( 5468, 1805, 0, 5592, 1841, 0, map, false );
				CreateTeleporter( 5551, 1805, 14, 5556, 1827, 0, map, false );
				CreateTeleporter( 5551, 1807, 0, 5551, 1807, 7, map, false );
				CreateTeleporter( 5552, 1805, 14, 5557, 1827, 0, map, false );
				CreateTeleporter( 5552, 1807, 0, 5552, 1807, 7, map, false );
				CreateTeleporter( 5553, 1805, 14, 5557, 1827, 0, map, false );
				CreateTeleporter( 5553, 1807, 0, 5552, 1807, 7, map, false );
				CreateTeleporter( 5556, 1825, -7, 5556, 1825, 0, map, false );
				CreateTeleporter( 5556, 1826, -12, 5556, 1826, 0, map, false );
				CreateTeleporter( 5556, 1827, -13, 5551, 1807, 0, map, false );
				CreateTeleporter( 5557, 1825, -7, 5557, 1825, 0, map, false );
				CreateTeleporter( 5557, 1826, -12, 5557, 1826, 0, map, false );
				CreateTeleporter( 5557, 1827, -13, 5552, 1807, 0, map, false );
				CreateTeleporter( 5558, 1825, -7, 5557, 1825, 0, map, false );
				CreateTeleporter( 5595, 1840, -14, 5468, 1806, 0, map, false );
				CreateTeleporter( 5595, 1841, -14, 5468, 1806, 0, map, false );

				// Wrong
				CreateTeleporter( 5824, 631, 0, 2041, 215, 14, map, true );
				CreateTeleporter( 5825, 631, 0, 2042, 215, 14, map, true );
				CreateTeleporter( 5826, 631, 0, 2043, 215, 14, map, true );
				CreateTeleporter( 5863, 525, 15, 5703, 639, 0, map, false );
				CreateTeleporter( 5863, 526, 15, 5703, 639, 0, map, false );
				CreateTeleporter( 5863, 527, 15, 5703, 639, 0, map, false );
				CreateTeleporter( 5868, 537, 15, 5690, 570, 25, map, false );
				CreateTeleporter( 5868, 538, 15, 5690, 570, 25, map, false );
				CreateTeleporter( 5869, 538, 15, 5690, 570, 25, map, false );
				CreateTeleporter( 5733, 554, 20, 5825, 593, 0, map, false );
				CreateTeleporter( 5862, 527, 15, 5703, 639, 0, map, false );
				CreateTeleporter( 5698, 662, 0, 5793, 527, 10, map, false );

				// Deceit
				CreateTeleporter( 4110, 430, 17, 5187, 639, 0, map, false );
				CreateTeleporter( 4111, 430, 17, 5188, 639, 0, map, false );
				CreateTeleporter( 4112, 430, 17, 5189, 639, 0, map, false );
				CreateTeleporter( 5187, 639, 0, 4110, 430, 5, map, false );
				CreateTeleporter( 5188, 639, 0, 4111, 430, 5, map, false );
				CreateTeleporter( 5189, 639, 0, 4112, 430, 5, map, false );
				CreateTeleporter( 5216, 586, -13, 5304, 533, 2, map, false );
				CreateTeleporter( 5217, 586, -13, 5305, 533, 2, map, false );
				CreateTeleporter( 5218, 586, -13, 5306, 533, 2, map, false );
				CreateTeleporter( 5304, 532, 7, 5216, 585, -8, map, false );
				CreateTeleporter( 5305, 532, 7, 5217, 585, -8, map, false );
				CreateTeleporter( 5306, 532, 7, 5218, 585, -8, map, false );
				CreateTeleporter( 5218, 761, -28, 5305, 651, 7, map, false );
				CreateTeleporter( 5219, 761, -28, 5306, 651, 7, map, false );
				CreateTeleporter( 5305, 650, 12, 5218, 760, -23, map, false );
				CreateTeleporter( 5306, 650, 12, 5219, 760, -23, map, false );
				CreateTeleporter( 5346, 578, 5, 5137, 649, 5, map, true );

				// Despise
				CreateTeleporter( 5504, 569, 46, 5574, 628, 37, map, false );
				CreateTeleporter( 5504, 570, 46, 5574, 629, 37, map, false );
				CreateTeleporter( 5504, 571, 46, 5574, 630, 37, map, false );
				CreateTeleporter( 5572, 632, 17, 5521, 672, 27, map, false );
				CreateTeleporter( 5572, 633, 17, 5521, 673, 27, map, false );
				CreateTeleporter( 5572, 634, 17, 5521, 674, 27, map, false );
				CreateTeleporter( 5573, 628, 42, 5503, 569, 51, map, false );
				CreateTeleporter( 5573, 629, 42, 5503, 570, 51, map, false );
				CreateTeleporter( 5573, 630, 42, 5503, 571, 51, map, false );
				CreateTeleporter( 5588, 632, 30, 1296, 1082, 0, map, true );
				CreateTeleporter( 5588, 630, 30, 1296, 1080, 0, map, true );
				CreateTeleporter( 5588, 631, 30, 1296, 1081, 0, map, true );
				CreateTeleporter( 5522, 672, 32, 5573, 632, 22, map, false );
				CreateTeleporter( 5522, 673, 32, 5573, 633, 22, map, false );
				CreateTeleporter( 5522, 674, 32, 5573, 634, 22, map, false );
				CreateTeleporter( 5386, 756, -8, 5408, 859, 47, map, false );
				CreateTeleporter( 5386, 757, -8, 5408, 860, 47, map, false );
				CreateTeleporter( 5386, 755, -8, 5408, 858, 47, map, false );
				CreateTeleporter( 5409, 860, 52, 5387, 757, -3, map, false );
				CreateTeleporter( 5409, 858, 52, 5387, 755, -3, map, false );
				CreateTeleporter( 5409, 859, 52, 5387, 756, -3, map, false );

				// Destard
				CreateTeleporter( 5242, 1007, 0, 1175, 2635, 0, map, true );
				CreateTeleporter( 5243, 1007, 0, 1176, 2635, 0, map, true );
				CreateTeleporter( 5244, 1007, 0, 1177, 2635, 0, map, true );
				CreateTeleporter( 5142, 797, 25, 5129, 908, -20, map, false );
				CreateTeleporter( 5143, 797, 25, 5130, 908, -20, map, false );
				CreateTeleporter( 5144, 797, 25, 5131, 908, -20, map, false );
				CreateTeleporter( 5145, 797, 25, 5132, 908, -20, map, false );
				CreateTeleporter( 5153, 808, -22, 5134, 984, 20, map, false );
				CreateTeleporter( 5153, 809, -22, 5134, 985, 20, map, false );
				CreateTeleporter( 5153, 810, -22, 5134, 986, 20, map, false );
				CreateTeleporter( 5153, 811, -22, 5134, 987, 20, map, false );
				CreateTeleporter( 5129, 909, -25, 5142, 798, 20, map, false );
				CreateTeleporter( 5130, 909, -25, 5143, 798, 20, map, false );
				CreateTeleporter( 5131, 909, -25, 5144, 798, 20, map, false );
				CreateTeleporter( 5132, 909, -25, 5145, 798, 20, map, false );
				CreateTeleporter( 5133, 984, 25, 5152, 808, -16, map, false );
				CreateTeleporter( 5133, 985, 25, 5152, 809, -16, map, false );
				CreateTeleporter( 5133, 986, 25, 5152, 810, -16, map, false );
				CreateTeleporter( 5133, 987, 25, 5152, 811, -16, map, false );

				// Misc
				CreateTeleporter( 5217, 18, 15, 5204, 74, 17, map, false );
				CreateTeleporter( 5200, 71, 17, 5211, 22, 15, map, false );
				CreateTeleporter( 1997, 81, 7, 5881, 242, -1, map, false );
				CreateTeleporter( 5704, 146, -45, 5705, 305, 7, map, false );
				CreateTeleporter( 5704, 147, -45, 5705, 306, 7, map, false );
				CreateTeleporter( 5874, 146, 30, 5208, 2323, 30, map, false );
				CreateTeleporter( 5875, 146, 30, 5208, 2322, 32, map, false );
				CreateTeleporter( 5923, 169, 1, 5925, 171, 22, map, false );
				CreateTeleporter( 2399, 198, 0, 5753, 436, 80, map, false );
				CreateTeleporter( 2400, 198, 0, 5754, 436, 80, map, false );
				CreateTeleporter( 5166, 245, 15, 1361, 883, 0, map, true );
				CreateTeleporter( 5849, 239, -25, 5831, 324, 33, map, false );
				CreateTeleporter( 5850, 239, -25, 5832, 324, 28, map, false );
				CreateTeleporter( 5851, 239, -25, 5833, 324, 33, map, false );
				CreateTeleporter( 5852, 239, -25, 5834, 324, 31, map, false );
				CreateTeleporter( 5853, 239, -25, 5835, 324, 33, map, false );
				CreateTeleporter( 5882, 241, 0, 1998, 81, 7, map, false );
				CreateTeleporter( 5882, 242, 0, 1998, 81, 7, map, false );
				CreateTeleporter( 5882, 243, 0, 1998, 81, 7, map, false );
				CreateTeleporter( 5706, 305, 12, 5705, 146, -45, map, false );
				CreateTeleporter( 5706, 306, 12, 5705, 147, -45, map, false );
				CreateTeleporter( 5748, 362, 3, 313, 786, -24, map, false );
				CreateTeleporter( 5749, 362, 2, 313, 786, -24, map, false );
				CreateTeleporter( 5750, 362, 5, 314, 786, -24, map, false );
				CreateTeleporter( 5753, 324, 28, 5670, 2391, 40, map, false );
				CreateTeleporter( 5831, 323, 35, 5849, 238, -25, map, false );
				CreateTeleporter( 5832, 323, 35, 5850, 238, -25, map, false );
				CreateTeleporter( 5833, 323, 34, 5851, 238, -25, map, false );
				CreateTeleporter( 5834, 323, 33, 5852, 238, -25, map, false );
				CreateTeleporter( 5835, 323, 35, 5853, 238, -25, map, false );
				CreateTeleporter( 5658, 423, 14, 5697, 3659, 2, map, false );
				CreateTeleporter( 5686, 385, 2, 2777, 894, -23, map, false );
				CreateTeleporter( 5686, 386, 3, 2777, 894, -23, map, false );
				CreateTeleporter( 5686, 387, 2, 2777, 895, -23, map, false );
				CreateTeleporter( 5731, 445, -20, 6087, 3676, 17, map, false );
				CreateTeleporter( 5753, 437, 80, 2400, 199, 0, map, false );
				CreateTeleporter( 5850, 432, 2, 5127, 3143, 98, map, false );
				CreateTeleporter( 5850, 433, 2, 5127, 3143, 98, map, false );
				CreateTeleporter( 5850, 434, -2, 5127, 3143, 98, map, false );
				CreateTeleporter( 5850, 431, 2, 5127, 3143, 98, map, false );
				CreateTeleporter( 5826, 465, 0, 1987, 2063, -40, map, false );
				CreateTeleporter( 5827, 465, -1, 1988, 2063, -40, map, false );
				CreateTeleporter( 5828, 465, -2, 1989, 2063, -40, map, false );
				CreateTeleporter( 313, 786, -24, 5748, 361, 0, map, false );
				CreateTeleporter( 314, 786, -24, 5749, 361, 2, map, false );
				CreateTeleporter( 2776, 895, -23, 5685, 387, 2, map, false );
				CreateTeleporter( 4545, 851, 30, 5736, 3196, 10, map, false );
				CreateTeleporter( 4540, 898, 32, 4442, 1122, 5, map, false );
				CreateTeleporter( 4300, 968, 5, 4442, 1122, 5, map, false );
				CreateTeleporter( 4436, 1107, 5, 4300, 992, 5, map, false );
				CreateTeleporter( 4443, 1137, 5, 4487, 1475, 5, map, false );
				CreateTeleporter( 4449, 1107, 5, 4539, 890, 28, map, false );
				CreateTeleporter( 4449, 1115, 5, 4671, 1135, 10, map, false );
				CreateTeleporter( 4663, 1134, 13, 4442, 1122, 5, map, false );
				CreateTeleporter( 5701, 1320, 16, 5786, 1336, -8, map, false );
				CreateTeleporter( 5702, 1320, 16, 5787, 1336, -8, map, false );
				CreateTeleporter( 5703, 1320, 16, 5788, 1336, -8, map, false );
				CreateTeleporter( 5786, 1335, -13, 5701, 1319, 13, map, false );
				CreateTeleporter( 5787, 1335, -13, 5702, 1319, 13, map, false );
				CreateTeleporter( 5788, 1335, -13, 5703, 1319, 13, map, false );
				CreateTeleporter( 6005, 1380, 3, 5151, 4063, 37, map, false );
				CreateTeleporter( 6005, 1378, -1, 5151, 4062, 37, map, false );
				CreateTeleporter( 6005, 1379, 0, 5151, 4062, 37, map, false );
				CreateTeleporter( 6025, 1344, -26, 5137, 3664, 30, map, false );
				CreateTeleporter( 6025, 1345, -25, 5137, 3664, 30, map, false );
				CreateTeleporter( 6025, 1346, -24, 5137, 3665, 30, map, false );
				CreateTeleporter( 5687, 1424, 38, 2923, 3406, 8, map, false );
				CreateTeleporter( 5792, 1416, 42, 5758, 2908, 16, map, false );
				CreateTeleporter( 5792, 1417, 42, 5758, 2909, 14, map, false );
				CreateTeleporter( 5792, 1415, 43, 5758, 2907, 16, map, false );
				CreateTeleporter( 5899, 1411, 43, 1630, 3320, 0, map, false );
				CreateTeleporter( 5900, 1411, 42, 1630, 3320, 0, map, false );
				CreateTeleporter( 5918, 1412, -28, 5961, 1409, 60, map, false );
				CreateTeleporter( 5918, 1410, -28, 5961, 1408, 60, map, false );
				CreateTeleporter( 5918, 1411, -28, 5961, 1408, 60, map, false );
				CreateTeleporter( 5961, 1408, 60, 5918, 1411, -28, map, false );
				CreateTeleporter( 5961, 1409, 60, 5918, 1412, -28, map, false );
				CreateTeleporter( 6125, 1411, 15, 6075, 3332, 3, map, false );
				CreateTeleporter( 6126, 1411, 15, 6075, 3332, 3, map, false );
				CreateTeleporter( 6127, 1411, 15, 6075, 3332, 3, map, false );
				CreateTeleporter( 6137, 1409, 2, 6140, 1432, 4, map, false );
				CreateTeleporter( 6138, 1409, 2, 6140, 1432, 4, map, false );
				CreateTeleporter( 6140, 1431, 4, 6137, 1408, 3, map, false );
				CreateTeleporter( 4496, 1475, 15, 4442, 1122, 5, map, false );
				CreateTeleporter( 6031, 1501, 42, 1491, 1642, 24, map, false );
				CreateTeleporter( 6031, 1499, 42, 1491, 1640, 24, map, false );
				CreateTeleporter( 1491, 1640, 24, 6032, 1499, 31, map, false );
				CreateTeleporter( 1491, 1642, 24, 6032, 1501, 31, map, false );
				CreateTeleporter( 5341, 1602, 0, 5426, 3122, -68, map, false );
				CreateTeleporter( 1987, 2062, -40, 5826, 464, 1, map, false );
				CreateTeleporter( 1988, 2062, -40, 5827, 464, 1, map, false );
				CreateTeleporter( 1989, 2062, -40, 5828, 464, -1, map, false );
				CreateTeleporter( 2666, 2073, 5, 2667, 2071, -20, map, false );
				CreateTeleporter( 2669, 2072, -20, 2666, 2074, 5, map, false );
				CreateTeleporter( 2669, 2073, -20, 2666, 2075, 5, map, false );
				CreateTeleporter( 2649, 2194, 4, 2649, 2196, -20, map, false );
				CreateTeleporter( 2649, 2195, -14, 2649, 2194, 3, map, false );
				CreateTeleporter( 5203, 2327, 28, 5876, 147, 25, map, false );
				CreateTeleporter( 5207, 2322, 28, 5877, 147, 25, map, false );
				CreateTeleporter( 5207, 2323, 27, 5876, 147, 25, map, false );
				CreateTeleporter( 5670, 2391, 40, 5753, 325, 3, map, false );
				CreateTeleporter( 5974, 2697, 35, 2985, 2890, 35, map, false );
				CreateTeleporter( 5267, 2757, 35, 424, 3283, 35, map, false );
				CreateTeleporter( 5757, 2908, 15, 5791, 1416, 38, map, false );
				CreateTeleporter( 5757, 2909, 15, 5791, 1417, 40, map, false );
				CreateTeleporter( 5757, 2907, 15, 5791, 1415, 39, map, false );
				CreateTeleporter( 1653, 2963, 0, 1677, 2987, 0, map, false );
				CreateTeleporter( 1677, 2987, 0, 1675, 2987, 20, map, false );
				CreateTeleporter( 5426, 3123, -80, 5341, 1602, 0, map, false );
				CreateTeleporter( 5126, 3143, 99, 5849, 432, 2, map, false );
				CreateTeleporter( 5736, 3196, 10, 4545, 851, 30, map, false );
				CreateTeleporter( 424, 3283, 35, 5267, 2757, 35, map, false );
				CreateTeleporter( 1629, 3320, 0, 5899, 1411, 43, map, false );
				CreateTeleporter( 6075, 3332, 3, 6126, 1410, 15, map, false );
				CreateTeleporter( 2923, 3405, 7, 5687, 1423, 41, map, false );
				CreateTeleporter( 1142, 3621, 5, 1414, 3828, 5, map, false );
				CreateTeleporter( 5137, 3664, 30, 6025, 1344, -26, map, false );
				CreateTeleporter( 5137, 3665, 30, 6025, 1345, -26, map, false );
				CreateTeleporter( 5697, 3660, 1, 5658, 424, -1, map, false );
				CreateTeleporter( 6086, 3676, 15, 5731, 446, -20, map, false );
				CreateTeleporter( 1409, 3824, 5, 1124, 3623, 5, map, false );
				CreateTeleporter( 1419, 3832, 5, 1466, 4015, 5, map, false );
				CreateTeleporter( 1406, 3996, 5, 1414, 3828, 5, map, false );
				CreateTeleporter( 5150, 4062, 40, 6005, 1378, -1, map, false );
				CreateTeleporter( 5150, 4063, 40, 6005, 1380, 0, map, false );
				CreateTeleporter( 5906, 4069, 26, 2494, 3576, 5, map, true );
				CreateTeleporter( 2985, 2890, 35, 5974, 2697, 35, map, false );
			}
			
			public int CreateTeleporters()
			{
				CreateTeleportersMap( Map.Felucca );
				CreateTeleportersMap( Map.Trammel );
				return m_Count;
			}
		}
	}
}
