using System;
using Server.Gumps;
using Server.Network;
using Server.Items;

namespace Server.Engines.Craft
{
	public class CraftGumpItem : Gump
	{
		private Mobile m_From;
		private CraftSystem m_CraftSystem;
		private CraftItem m_CraftItem;
		private int m_SelectedGroup;
		private int m_GroupPage;
		private int m_ItemPage;
		private int m_ResIndex;
		private BaseTool m_Tool;

		public CraftGumpItem( Mobile from, CraftSystem craftSystem, CraftItem craftItem, int selectedGroup, int groupPage, int itemPage, int resIndex, BaseTool tool ) : base( 40, 40 )
		{
			m_From = from;
			m_CraftSystem = craftSystem;
			m_CraftItem = craftItem;
			m_SelectedGroup = selectedGroup;
			m_GroupPage = groupPage;
			m_ItemPage = itemPage;
			m_ResIndex = resIndex;
			m_Tool = tool;

			AddPage( 0 );
			AddBackground( 0, 0, 531, 417, 5054 );
			AddImageTiled( 10, 10, 511, 397, 2624 );

			AddImageTiled( 10, 32, 511, 5, 5058 );
			
			AddImageTiled( 160, 37, 5, 148, 5058 );	//    |
			AddImageTiled( 165, 127, 356, 5, 5058 );// -----------
			AddImageTiled( 10, 185, 155, 5, 5058 );	// ----

			
			AddImageTiled( 10, 212, 511, 5, 5058 );	// -------------------
			AddImageTiled( 160, 217, 5, 53, 5058 );	//      |
			AddImageTiled( 10, 270, 155, 5, 5058 );	// ------


			AddImageTiled( 10, 317, 511, 5, 5058 );	// -------------------
			AddImageTiled( 160, 322, 5, 33, 5058 );	//      |
			AddImageTiled( 10, 355, 155, 5, 5058 );	// ------

			AddImageTiled( 10, 382, 511, 5, 5058 );	// -------------------

			AddAlphaRegion( 10, 10, 511, 397 );

			AddLabel( craftSystem.GumpTitlePos, 10, 0x384, craftSystem.GumpTitle );
			AddLabel( 167, 39, 0x384, "ITEM" );
			AddLabel( 167, 79, 0x384, "Success Chance:" );
			AddLabel( 50, 192, 0x384, "SKILLS" );
			AddLabel( 30, 277, 0x384, "MATERIALS" );
			AddLabel( 55, 362, 0x384, "OTHER" );

			AddButton( 15, 387, 0xFAE, 0xFB0, 0, GumpButtonType.Reply, 0 );
			AddLabel( 45, 387, 0x384, "BACK" );

			AddButton( 270, 387, 0xFA5, 0xFA7, 1, GumpButtonType.Reply, 0 );
			AddLabel( 300, 387, 0x384, "MAKE NOW" );

			AddLabel( 327, 39, 0x384, craftItem.ItemName );

			DrawItem();
			DrawSkill();
			DrawRessource();
		}

		public void DrawItem()
		{
			Item item = Activator.CreateInstance( m_CraftItem.ItemType ) as Item;

			if ( item != null )
			{
				AddItem( 20, 47, item.ItemID );

				item.Delete();
			}
		}

		public void DrawSkill()
		{
			double tempChance = 0;
			double chance = 100;
			for ( int i = 0; i < m_CraftItem.Skills.Count; i++ )
			{
				AddLabel( 167, ( 20 * i ) + 131, 0x384, m_CraftItem.Skills.GetAt( i ).SkillToMake.ToString() );
				
				double skillValue = m_From.Skills[m_CraftItem.Skills.GetAt( i ).SkillToMake].Value;
				skillValue = Math.Round( skillValue, 1 );
				AddLabel( 378, ( 20 * i ) + 131, 0x384, skillValue.ToString() );


				double minSkill = m_CraftItem.Skills.GetAt( i ).MinSkill;
				double maxSkill = m_CraftItem.Skills.GetAt( i ).MaxSkill;
				
				if ( m_CraftItem.Skills.GetAt(i).SkillToMake == m_CraftSystem.MainSkill )
				{
					Type type = null;
					if ( m_ResIndex > -1 )
					{
						type = m_CraftSystem.CraftSubRes.GetAt( m_ResIndex ).ItemType;
						CraftItem.AdjustSkillsValue( m_From, type, m_CraftSystem, ref minSkill, ref maxSkill );
					}
				}

				tempChance = skillValue - minSkill;
				tempChance = tempChance / ( maxSkill - minSkill );

				if ( tempChance < chance )
					chance = tempChance;
				
				AddLabel( 414, ( 20 * i ) + 131, 0x384, "/" );
				AddLabel( 427, ( 20 * i ) + 131, 0x384, Math.Round( minSkill, 1 ).ToString() );
			}

			chance *= 100;
			if ( chance > 100 )
				chance = 100;
			else if ( chance < 0 )
				chance = 0;
			else
				chance = Math.Round( chance );
				
			AddLabel( 378, 79, 0x384, string.Format( "{0}%", chance ) );
		}

		public void DrawRessource()
		{
			for ( int i = 0; i < m_CraftItem.Ressources.Count; i++ )
			{
				Type type = m_CraftItem.Ressources.GetAt( i ).ItemType;
				string name = m_CraftItem.Ressources.GetAt( i ).ItemName;
				
				// Resource Mutation
				if ( type == m_CraftSystem.CraftSubRes.ResType && m_ResIndex > -1 )
					name = m_CraftSystem.CraftSubRes.GetAt( m_ResIndex ).ItemName;
				// ******************

				AddLabel( 167, ( 20 * i ) + 216, 0x384, name );
				AddLabel( 378, ( 20 * i ) + 216, 0x384, Convert.ToString( m_CraftItem.Ressources.GetAt( i ).Amount ) );
			}

			if ( m_CraftItem.ItemName == "Runebook" )
			{
				AddLabel( 167, ( 20 * m_CraftItem.Ressources.Count ) + 216, 0x384, "Recall Rune" );
				AddLabel( 378, ( 20 * m_CraftItem.Ressources.Count ) + 216, 0x384, "8" );
			}

		}

		public override void OnResponse( NetState sender, RelayInfo info )
		{
			// Back Button
			if ( info.ButtonID == 0 )
			{
				CraftGump craftGump = new CraftGump( m_From, m_CraftSystem, m_SelectedGroup, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGump );
			}

			else // Make Button
			{
				if ( m_CraftSystem.CanCraft( m_From, m_Tool, m_CraftItem.ItemType ) )
				{
					Type type = null;
					if ( m_ResIndex > -1 )
					{
						type = m_CraftSystem.CraftSubRes.GetAt( m_ResIndex ).ItemType;
					}
					m_CraftSystem.CreateItem( m_From, m_CraftItem.ItemType, type, m_Tool );
				}

				CraftGumpItem craftGumpItem = new CraftGumpItem( m_From, m_CraftSystem, m_CraftItem, m_SelectedGroup, m_GroupPage, m_ItemPage, m_ResIndex, m_Tool );
				m_From.SendGump( craftGumpItem );
			}
		}
	}
}