using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public enum ConsumeType
	{
		All, Half, None
	}

	public class CraftItem
	{
		private CraftResCol m_arCraftRes;
		private CraftSkillCol m_arCraftSkill;
		private Type m_Type;
		private string m_strGroup;
		private string m_strName;
		
		private int m_Mana;
		private int m_Hits;
		private int m_Stam;

		private bool m_UseAllRes = false;

		public CraftItem( Type type, string strGroup, string strName )
		{
			m_arCraftRes = new CraftResCol();
			m_arCraftSkill = new CraftSkillCol();

			m_Type = type;
			m_strGroup = strGroup;
			m_strName = strName;
		}

		public void AddRes( Type type, string name, int amount )
		{
			AddRes( type, name, amount, "" );
		}

		public void AddRes( Type type, string name, int amount, int localizedMessage )
		{
			CraftRes craftRes = new CraftRes( type, name, amount, localizedMessage );
			m_arCraftRes.Add( craftRes );
		}

		public void AddRes( Type type, string name, int amount, string strMessage )
		{
			CraftRes craftRes = new CraftRes( type, name, amount, strMessage );
			m_arCraftRes.Add( craftRes );
		}

		public void AddSkill( SkillName skillToMake, double minSkill, double maxSkill )
		{
			CraftSkill craftSkill = new CraftSkill( skillToMake, minSkill, maxSkill );
			m_arCraftSkill.Add( craftSkill );
		}

		public int Mana
		{
			get { return m_Mana; }
			set { m_Mana = value; }
		}

		public int Hits
		{
			get { return m_Hits; }
			set { m_Hits = value; }
		}

		public int Stam
		{
			get { return m_Stam; }
			set { m_Stam = value; }
		}

		public bool UseAllRes
		{
			get { return m_UseAllRes; }
			set { m_UseAllRes = value; }
		}
		
		public Type ItemType
		{
			get { return m_Type; }
		}

		public string ItemGroup
		{
			get { return m_strGroup; }
		}

		public string ItemName
		{
			get { return m_strName; }
		}

		public CraftResCol Ressources
		{
			get { return m_arCraftRes; }
		}

		public CraftSkillCol Skills
		{
			get { return m_arCraftSkill; }
		}

		public bool ConsumeAttributes( Mobile from )
		{
			bool consumMana = false;
			bool consumHits = false;
			bool consumStam = false;
			
			if ( Hits > 0 && from.Hits <= Hits )
			{
				from.SendMessage( "You lack the hits required to craft that item" );
				return false;
			}
			else
				consumHits = true;

			if ( Mana > 0 && from.Mana <= Mana )
			{
				from.SendMessage( "You lack the mana required to craft that item" );
				return false;
			}
			else
				consumMana = true;
			
			if ( Stam > 0 && from.Stam <= Stam )
			{
				from.SendMessage( "You are to tired to craft that item" );
				return false;
			}
			else
				consumStam = true;


			if ( consumMana )
				from.Mana -= Mana;

			if ( consumHits )
				from.Hits -= Hits;

			if ( consumStam )
				from.Stam -= Stam;

			return true;
		}

		public bool ConsumeRes( Mobile from, Type typeRes, CraftSystem craftSystem, ref int resHue, ref int maxAmount, ConsumeType consumeType )
		{
			Container ourPack = from.Backpack;

			if ( ourPack == null )
				return false;

			Type[] types = new Type[m_arCraftRes.Count];
			int[] amounts = new int[m_arCraftRes.Count];

			maxAmount = int.MaxValue;

			for ( int i = 0; i < types.Length; ++i )
			{
				CraftRes craftRes = m_arCraftRes.GetAt( i );

				types[i] = craftRes.ItemType;

				// Get the color of the first resource in the list
				if ( i == 0 )
				{
					Item itemColor = ourPack.FindItemByType( craftRes.ItemType );
					if ( itemColor != null )
					{
						if ( craftRes.Amount <= itemColor.Amount )
							resHue = itemColor.Hue;
					}
				}
				// ********

				// Resource Mutation
				if ( ( types[i] == craftSystem.CraftSubRes.ResType ) && ( typeRes != null ) )
					types[i] = typeRes;
				// ******************

				amounts[i] = craftRes.Amount;

				// For stackable items that can ben crafted more than one at a time
				if ( UseAllRes )
				{
					int tempAmount = GetTotalResAmount( ourPack, types[i] );
					tempAmount /= amounts[i];
					if ( tempAmount < maxAmount )
					{
						maxAmount = tempAmount;
					}
				}
				// ****************************

			}

			// We adjust the amount of each resource to consume the max posible
			if ( UseAllRes )
			{
				for ( int i = 0; i < amounts.Length; ++i )
				{
					amounts[i] *= maxAmount;
				}
			}
			else
				maxAmount = -1;

			Item consumeExtra = null;

			if ( m_strName == "Runebook" )
			{
				// Runebooks are a special case, they need a blank recall rune

				Item[] runes = ourPack.FindItemsByType( typeof( RecallRune ) );

				for ( int i = 0; i < runes.Length; ++i )
				{
					RecallRune rune = runes[i] as RecallRune;

					if ( rune != null && !rune.Marked )
					{
						consumeExtra = rune;
						break;
					}
				}

				if ( consumeExtra == null )
				{
					from.SendMessage( "You need an unmarked recall rune to craft this." );
					return false;
				}
			}
			
			int index = 0;
			
			// Consume ALL
			if ( consumeType == ConsumeType.All )
			{
				index = ourPack.ConsumeTotal( types, amounts, true );
			}

			// Consume Half ( for use all resource craft type )
			else if ( consumeType == ConsumeType.Half )
			{
				for ( int i = 0; i < amounts.Length; i++ )
					amounts[i] /= 2;

				index = ourPack.ConsumeTotal( types, amounts, true );
			}

			else // ConstumeType.None ( it's basicaly used to know if the crafter has enough resource before starting the process )
			{
				for ( int i = 0; i < types.Length; i++ )
				{
					if ( ourPack.GetAmount( types[i], true ) >= amounts[i] )
					{
						index = -1;
						break;
					}
				}
			}

			if ( index == -1 )
			{
				if ( consumeType != ConsumeType.None )
					if ( consumeExtra != null )
						consumeExtra.Delete();

				return true;
			}
			else
			{
				m_arCraftRes.GetAt(index).SendMessage( from );
				return false;
			}
		}

		public int GetTotalResAmount( Container pack, Type type )
		{
			Item[] items = pack.FindItemsByType( type );

			int totalAmount = 0;
			for ( int i = 0; i < items.Length; i++ )
				totalAmount += items[i].Amount;

			return totalAmount;
		}

		public bool SkillCheck( Mobile from, Type typeRes, CraftSystem craftSystem, ref int quality )
		{
			bool bCheck = true;
			
			double minSkill = 0;
			double maxSkill = 0;

			for (int i = 0;  i < m_arCraftSkill.Count; i++)
			{
				CraftSkill craftSkill = m_arCraftSkill.GetAt(i);

				minSkill = craftSkill.MinSkill;
				maxSkill = craftSkill.MaxSkill;

				if ( craftSkill.SkillToMake == craftSystem.MainSkill )
				{
					if ( typeRes != null )
					{
						AdjustSkillsValue( from, typeRes, craftSystem, ref minSkill, ref maxSkill );
					}
					
					quality = EvalQuality( from, minSkill, maxSkill, craftSkill.SkillToMake );
				}
				
				if ( !from.CheckSkill( craftSkill.SkillToMake, minSkill, maxSkill ) )
				{
					// We do not return here.  Because we need to train all the other skill in the collection.
					bCheck = false;
				}
			}

			// If at least one check has failed, general check failed
			return bCheck;
		}

		// Depending of the level of the selected resource, ajust the MinSkill and the MaxSkill for the CheckSkill
		public static void AdjustSkillsValue( Mobile from, Type typeRes, CraftSystem craftSystem, ref double minSkill, ref double maxSkill )
		{
			CraftSubRes craftSubRes = craftSystem.CraftSubRes.SearchFor( typeRes );

			double level = craftSubRes.ItemLevel;

			double minSkillValue = minSkill;
			minSkillValue /= 4;
			double minSkillAdjusted = minSkillValue * craftSubRes.ItemLevel;
			minSkill += ( minSkillAdjusted - minSkillValue );

			double maxSkillValue = maxSkill;
			maxSkillValue /= 4;
			double maxSkillAdjusted = maxSkillValue * craftSubRes.ItemLevel;
			maxSkill += ( maxSkillAdjusted - maxSkillValue );

			if ( maxSkill > 100 )
				maxSkill = 100;

			if ( minSkill > 100 )
				minSkill = 100;
		}

		// Will determine if the crafted item is Low, Regular or Exceptional
		public int EvalQuality(Mobile from, double minSkill, double maxSkill, SkillName skillName )
		{
			Skill skill = from.Skills[skillName];

			double value = skill.Value;
			double chance = (value - minSkill) / (maxSkill - minSkill);
			double random = Utility.RandomDouble();
			double result = chance + random;
			
			// Regular Quality
			int quality = 1;

			if ( result > 1.8 )
			{
				// Exceptional Quality
				quality = 2;
			}
			else if ( result < 0.8 )
			{
				// Low Quality
				quality = 0;
			}

			return quality;
		}

		public void Craft( Mobile from, CraftSystem craftSystem, Type typeRes, BaseTool tool )
		{
			if ( from.BeginAction( typeof( CraftSystem ) ) )
			{
				int resHue = 0;
				int maxAmount = 0;

				if ( ConsumeRes( from, typeRes, craftSystem, ref resHue, ref maxAmount, ConsumeType.None ) )
				{
					if ( ConsumeAttributes( from ) )
					{
						int iMin = craftSystem.MinCraftEffect;
						int iMax = ( craftSystem.MaxCraftEffect - iMin ) + 1;
						int iRandom = Utility.Random( iMax );
						iRandom += iMin + 1;
						new InternalTimer( from, craftSystem, this, typeRes, tool, iRandom, resHue, maxAmount ).Start();
					}
					else
					{
						from.EndAction( typeof( CraftSystem ) );
					}
				}
				else
				{
					from.EndAction( typeof( CraftSystem ) );
				}
			}
			else
			{
				from.SendLocalizedMessage( 500119 ); // You must wait to perform another action
			}
		}

		private class InternalTimer : Timer
		{
			private Mobile m_From;
			private int m_iCount;
			private int m_iCountMax;
			private CraftItem m_CraftItem;
			private CraftSystem m_CraftSystem;
			private Type m_TypeRes;
			private BaseTool m_Tool;
			private int m_Hue;
			private int m_MaxAmount;

			public InternalTimer( Mobile from, CraftSystem craftSystem, CraftItem craftItem, Type typeRes, BaseTool tool, int iCountMax, int hue, int maxAmount ) : base( TimeSpan.Zero, TimeSpan.FromSeconds( craftSystem.Delay ), iCountMax )
			{
				m_From = from;
				m_CraftItem = craftItem;
				m_iCount = 0;
				m_iCountMax = iCountMax;
				m_CraftSystem = craftSystem;
				m_TypeRes = typeRes;
				m_Tool = tool;
				m_Hue = hue;
				m_MaxAmount = maxAmount;
			}

			protected override void OnTick()
			{
				m_iCount++;

				if ( m_iCount < m_iCountMax )
				{
					m_CraftSystem.PlayCraftEffect( m_From );
				}
				else
				{
					m_From.EndAction( typeof( CraftSystem ) );

					m_Tool.Durability--;

					bool toolBroken = false;
					if ( m_Tool.Durability < 1 )
						toolBroken = true;

					int quality = 1;
					int endquality = 1;

					if ( m_CraftItem.SkillCheck( m_From, m_TypeRes, m_CraftSystem, ref quality ) )
					{
						// Resource
						ConsumeType consumeType = ( m_CraftItem.UseAllRes ? ConsumeType.Half : ConsumeType.All );
						int resHue = 0;
						int maxAmount = 0;

						// Not enough resource to craft it
						if ( !m_CraftItem.ConsumeRes( m_From, m_TypeRes, m_CraftSystem, ref resHue, ref maxAmount, ConsumeType.All ) )
						{
							return;
						}

						

						Item item = Activator.CreateInstance( m_CraftItem.ItemType ) as Item;

						if ( item != null )
						{
							// Special Trick for Blacksmith and colored Ingot
							// if TypeRes is null, means default material
							if ( item is BaseWeapon )
							{
								BaseWeapon weapon = ( BaseWeapon )item;
								weapon.Quality = (WeaponQuality)quality;
								endquality = quality;
								if ( quality == 2)
									weapon.Crafter = m_From;

								weapon.Hue = m_Hue;
							}
							else if ( item is BaseArmor )
							{
								BaseArmor armor = ( BaseArmor )item;
								armor.Quality = (ArmorQuality)quality;
								endquality = quality;
								if ( quality == 2)
									armor.Crafter = m_From;

								if ( m_TypeRes != null )
								{
									Item itemRes = Activator.CreateInstance( m_TypeRes ) as Item;

									OreInfo oreInfo = null;
									if ( itemRes is BaseIngot )
									{						
										BaseIngot ingot = (BaseIngot)itemRes;
										oreInfo = ingot.OreInfo;
									}

									if ( itemRes is BaseLeather )
									{						
										BaseLeather leather = (BaseLeather)itemRes;
										oreInfo = leather.OreInfo;
									}

									armor.OreType = oreInfo;
									armor.Hue = oreInfo.Hue;

								}
							}
							else if ( item.Hue == 0 )
							{
								item.Hue = m_Hue;
							}

							if ( m_MaxAmount > 0 )
							{
								item.Amount = m_MaxAmount;
							}

							// **********************************************
							
							m_From.AddToBackpack( item );
							m_From.PlaySound( 0x57 );
						}
						m_CraftSystem.PlayEndingEffect( m_From, false, true, toolBroken, endquality );
					}
					else
					{
						ConsumeType consumeType = ( m_CraftItem.UseAllRes ? ConsumeType.Half : ConsumeType.All );
						int resHue = 0;
						int maxAmount = 0;

						// Not enough resource to craft it
						if ( !m_CraftItem.ConsumeRes( m_From, m_TypeRes, m_CraftSystem, ref resHue, ref maxAmount, consumeType ) )
						{
							return;
						}
						
						// SkillCheck failed.
						m_CraftSystem.PlayEndingEffect( m_From, true, true, toolBroken, endquality );
					}
				}
			}
		}
	}
}