using System;

namespace Server.Items
{
	public abstract class Beverage : Item
	{
		private Poison m_Poison;
		private int m_AlcoholFactor;
		private int m_FillFactor;

		[CommandProperty( AccessLevel.GameMaster )]
		public Poison Poison
		{
			get { return m_Poison; }
			set { m_Poison = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int FillFactor
		{
			get { return m_FillFactor; }
			set { m_FillFactor = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public int AlcoholFactor
		{
			get { return m_AlcoholFactor; }
			set { m_AlcoholFactor = value; }
		}

		public Beverage( int itemID ) : base( itemID )
		{
			this.FillFactor = 1;
		}

		public Beverage( Serial serial ) : base( serial )
		{
		}

		public void Drink( Mobile from )
		{
			if ( FillThirst( from, m_FillFactor ) && ( FillBAC( from, m_AlcoholFactor ) ) )
			{
				// Play a random "drink" sound
				from.PlaySound( Utility.Random( 0x30, 2 ) );

				if ( from.Body.IsHuman && !from.Mounted )
					from.Animate( 34, 5, 1, true, false, 0 );

				if ( m_Poison != null )
					from.Poison = m_Poison;

				this.Consume();
			}
		}

		static public bool FillThirst( Mobile from, int fillFactor )
		{
			/*
			if ( from.Hunger >= 20 )
			{
				from.SendLocalizedMessage( 500867 ); // You are simply too full to eat any more!
				return false;
			}
			
			int iHunger = from.Hunger + fillFactor;
			if ( iHunger >= 20 )
			{
				from.Hunger = 20;
				from.SendLocalizedMessage( 500872 ); // You manage to eat the food, but you are stuffed!
			}
			else
			{
				from.Hunger = iHunger;

				if ( iHunger < 5 )
					from.SendLocalizedMessage( 500868 ); // You eat the food, but are still extremely hungry.
				else if ( iHunger < 10 )
					from.SendLocalizedMessage( 500869 ); // You eat the food, and begin to feel more satiated.
				else if ( iHunger < 15 )
					from.SendLocalizedMessage( 500870 ); // After eating the food, you feel much less hungry.
				else
					from.SendLocalizedMessage( 500871 ); // You feel quite full after consuming the food.
			}
			*/

			return true;
		}

		static public bool FillBAC( Mobile from, int alcoholFactor )
		{
			return true;
		}

		public override void OnDoubleClick( Mobile from )
		{
			if ( !Movable )
				return;

			if ( from.InRange( this.GetWorldLocation(), 1 ) )
			{
				Drink( from );
			}
		}

        public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
			Poison.Serialize( m_Poison, writer );
			writer.Write( m_FillFactor );
			writer.Write( m_AlcoholFactor );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Poison = Poison.Deserialize( reader );
					m_FillFactor = reader.ReadInt();
					m_AlcoholFactor = reader.ReadInt();
					break;
				}
			}
		}
	}

	[FlipableAttribute( 0x1f7d, 0x1f7e, 0x1f7f, 0x1f80 )]
	public class GlassCider : Beverage
	{
		[Constructable]
		public GlassCider() : base( 0x1f7d )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public GlassCider( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f85, 0x1f86, 0x1f87, 0x1f88 )]
	public class GlassLiquor : Beverage
	{
		[Constructable]
		public GlassLiquor() : base( 0x1f85 )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public GlassLiquor( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f89, 0x1f8a, 0x1f8b, 0x1f8c )]
	public class GlassMilk : Beverage
	{
		[Constructable]
		public GlassMilk() : base( 0x1f89 )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
		}

		public GlassMilk( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f8d, 0x1f8e, 0x1f8f, 0x1f90 )]
	public class GlassWine : Beverage
	{
		[Constructable]
		public GlassWine() : base( 0x1f8d )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public GlassWine( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f91, 0x1f92, 0x1f93, 0x1f94 )]
	public class GlassWater : Beverage
	{
		[Constructable]
		public GlassWater() : base( 0x1f91 )
		{
			this.Weight = 0.2;
			this.FillFactor = 1;
		}

		public GlassWater( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f95, 0x1f96 )]
	public class PitcherAle : Beverage
	{
		[Constructable]
		public PitcherAle() : base( 0x1f95 )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 3;
		}

		public PitcherAle( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f97, 0x1f98 )]
	public class PitcherCider : Beverage
	{
		[Constructable]
		public PitcherCider() : base( 0x1f97 )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 5;
		}

		public PitcherCider( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f99, 0x1f9a )]
	public class PitcherLiquor : Beverage
	{
		[Constructable]
		public PitcherLiquor() : base( 0x1f99 )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 5;
		}

		public PitcherLiquor( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x1f9b, 0x1f9c )]
	public class PitcherWine : Beverage
	{
		[Constructable]
		public PitcherWine() : base( 0x1f9b )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 5;
		}

		public PitcherWine( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	[FlipableAttribute( 0x9ee, 0x9ef )]
	public class MugAle : Beverage
	{
		[Constructable]
		public MugAle() : base( 0x9ee )
		{
			this.Weight = 1.0;
			this.FillFactor = 1;
			this.AlcoholFactor = 1;
		}

		public MugAle( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BottleWine : Beverage
	{
		[Constructable]
		public BottleWine() : base( 0x9c7 )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 5;
		}

		public BottleWine( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BottleAle : Beverage
	{
		[Constructable]
		public BottleAle() : base( 0x99f )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 3;
		}

		public BottleAle( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}

	public class BottleLiquor : Beverage
	{
		[Constructable]
		public BottleLiquor() : base( 0x99b )
		{
			this.Weight = 1.0;
			this.FillFactor = 2;
			this.AlcoholFactor = 5;
		}

		public BottleLiquor( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}