using System;
using Server;
using Server.Items;
using Server.Engines.Harvest;

namespace Server.Items
{
	public abstract class BasePoleArm : BaseMeleeWeapon
	{
		public BasePoleArm( int minDamage, int maxDamage, int speed, int hitSound, int missSound, int maxRange, SkillName skill, WeaponType type, WeaponAnimation animation, int itemID, Layer layer )
			: base( minDamage, maxDamage, speed, hitSound, missSound, maxRange, skill, type, animation, itemID, layer )
		{
		}

		public BasePoleArm( Serial serial ) : base( serial )
		{
		}

		public override void OnDoubleClick( Mobile from )
		{
			DefLumberjacking.HarvestSystem.Begin( from, this );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public override void OnHit( Mobile attacker, Mobile defender )
		{
			base.OnHit( attacker, defender );

			if ( Layer == Layer.TwoHanded && (attacker.Skills[SkillName.Anatomy].Value / 400.0) >= Utility.RandomDouble() )
			{
				StatMod mod = defender.GetStatMod( "Concussion" );

				if ( mod == null )
				{
					defender.SendMessage( "You receive a concussion blow!" );
					defender.AddStatMod( new StatMod( StatType.Int, "Concussion", -(attacker.RawInt / 2), TimeSpan.FromSeconds( 30.0 ) ) );

					attacker.SendMessage( "You deliver a concussion blow!" );
					attacker.PlaySound( 0x11C );
				}
			}
		}
	}
}