using System;
using Server;
using Server.Network;
using Server.Accounting;

namespace Server.Misc
{
	public class AccountHandler
	{
		private static bool AutoAccountCreation = true;

		public static void Initialize()
		{
			EventSink.AccountLogin += new AccountLoginEventHandler( EventSink_AccountLogin );
		}

		public static void EventSink_AccountLogin( AccountLoginEventArgs e )
		{
			string un = e.Username;
			string pw = e.Password;

			Account acct = Accounts.GetAccount( un );

			if ( acct == null )
			{
				if ( AutoAccountCreation )
				{
					Console.WriteLine( "Login: {0}: Creating new account '{1}'", e.State, un );
					e.State.Account = Accounts.AddAccount( un, pw );
					e.Accepted = true;
				}
				else
				{
					Console.WriteLine( "Login: {0}: Invalid username '{1}'", e.State, un );
					e.RejectReason = ALRReason.Invalid;
				}
			}
			else if ( acct.Password != pw )
			{
				Console.WriteLine( "Login: {0}: Invalid password for '{1}'", e.State, un );
				e.RejectReason = ALRReason.BadPass;
			}
			else if ( acct.Banned )
			{
				Console.WriteLine( "Login: {0}: Banned account '{1}'", e.State, un );
				e.RejectReason = ALRReason.Blocked;
			}
			else
			{
				Console.WriteLine( "Login: {0}: Valid credentials for '{1}'", e.State, un );
				e.State.Account = acct;
				e.Accepted = true;

				acct.LastLogin = DateTime.Now;
			}
		}
	}
}