using System;
using Server;
using Server.Items;
using Server.Mobiles;
using Server.Network;
using Server.Accounting;

namespace Server.Misc
{
	public class CharacterCreation
	{
		public static void Initialize()
		{
			// Register our event handler
			EventSink.CharacterCreated += new CharacterCreatedEventHandler( EventSink_CharacterCreated );
		}

		private static void AddBackpack( Mobile m )
		{
			// Construct backpack
			Backpack pack = new Backpack();

			pack.Movable = false;

			pack.DropItem( MakeNewbie( new RedBook( "a book", m.Name, 20, true ) ) );
			pack.DropItem( MakeNewbie( new Gold( 1000 ) ) ); // Starting gold can be customized here
			pack.DropItem( MakeNewbie( new Dagger() ) );

			m.AddItem( pack );
		}

		private static Item MakeNewbie( Item item )
		{
			item.LootType = LootType.Newbied;

			return item;
		}

		private static void FillBankbox( Mobile m )
		{
			BankBox bank = m.BankBox;

			if ( bank != null )
			{
				bank.DropItem( new BankCheck( 1000000 ) );

				// Full spellbook
				Spellbook book = new Spellbook();

				book.Content = ulong.MaxValue;

				bank.DropItem( book );

				// Bag containing 50 of each reagent
				bank.DropItem( new BagOfReagents( 50 ) );

				// Add tools here as they are implemented
				bank.DropItem( new Scissors() );

				// A few dye tubs
				bank.DropItem( new Dyes() );
				bank.DropItem( new DyeTub() );
				bank.DropItem( new DyeTub() );
				bank.DropItem( new BlackDyeTub() );

				DyeTub darkRedTub = new DyeTub();

				darkRedTub.DyedHue = 0x485;
				darkRedTub.Redyable = false;

				bank.DropItem( darkRedTub );

				// Some food
				bank.DropItem( new Apple( 1000 ) );

				// Resources
				bank.DropItem( new Feather( 1000 ) );
				bank.DropItem( new BoltOfCloth( 1000 ) );
				bank.DropItem( new BlankScroll( 1000 ) );
				bank.DropItem( new Hides( 1000 ) );
				bank.DropItem( new Bandage( 1000 ) );
				bank.DropItem( new Bottle( 1000 ) );
				bank.DropItem( new Log( 1000 ) );

				bank.DropItem( new IronIngot( 5000 ) );
				bank.DropItem( new DullCopperIngot( 5000 ) );
				bank.DropItem( new ShadowIronIngot( 5000 ) );
				bank.DropItem( new CopperIngot( 5000 ) );
				bank.DropItem( new BronzeIngot( 5000 ) );
				bank.DropItem( new GoldIngot( 5000 ) );
				bank.DropItem( new AgapiteIngot( 5000 ) );
				bank.DropItem( new VeriteIngot( 5000 ) );
				bank.DropItem( new ValoriteIngot( 5000 ) );

				// Reagents
				bank.DropItem( new BlackPearl( 1000 ) );
				bank.DropItem( new Bloodmoss( 1000 ) );
				bank.DropItem( new Garlic( 1000 ) );
				bank.DropItem( new Ginseng( 1000 ) );
				bank.DropItem( new MandrakeRoot( 1000 ) );
				bank.DropItem( new Nightshade( 1000 ) );
				bank.DropItem( new SulfurousAsh( 1000 ) );
				bank.DropItem( new SpidersSilk( 1000 ) );

				// Some extra starting gold
				bank.DropItem( new Gold( 9000 ) );

				// 5 blank recall runes
				for ( int i = 0; i < 5; ++i )
				{
					bank.DropItem( new RecallRune() );
				}

				AddPowerScrolls( bank );
			}
		}

		private static void AddPowerScrolls( BankBox bank )
		{
			Bag bag = new Bag();

			bag.DropItem( new PowerScroll( SkillName.Blacksmith, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Tailoring, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Swords, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Fencing, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Macing, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Archery, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Wrestling, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Parry, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Tactics, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Anatomy, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Healing, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Magery, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Meditation, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.EvalInt, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.MagicResist, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.AnimalTaming, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.AnimalLore, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Veterinary, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Musicianship, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Provocation, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Discordance, 120.0 ) );
			bag.DropItem( new PowerScroll( SkillName.Peacemaking, 120.0 ) );

			bag.DropItem( new StatCapScroll( 250 ) );

			bank.DropItem( bag );
		}

		private static void AddShirt( Mobile m, int shirtHue )
		{
			int hue = Utility.ClipDyedHue( shirtHue & 0x3FFF );

			switch ( Utility.Random( 2 ) )
			{
				case 0: m.AddItem( MakeNewbie( new Shirt( hue ) ) ); break;
				case 1: m.AddItem( MakeNewbie( new FancyShirt( hue ) ) ); break;
			}
		}

		private static void AddPants( Mobile m, int pantsHue )
		{
			int hue = Utility.ClipDyedHue( pantsHue & 0x3FFF );

			if ( m.Female )
			{
				m.AddItem( MakeNewbie( new Skirt( hue ) ) );
			}
			else
			{
				switch ( Utility.Random( 2 ) )
				{
					case 0: m.AddItem( MakeNewbie( new LongPants( hue ) ) ); break;
					case 1: m.AddItem( MakeNewbie( new ShortPants( hue ) ) ); break;
				}
			}
		}

		private static void AddShoes( Mobile m )
		{
			m.AddItem( MakeNewbie( new Shoes( Utility.RandomYellowHue() ) ) );
		}

		private static void AddHair( Mobile m, int itemID, int hue )
		{
			Item item;

			switch ( itemID & 0x3FFF )
			{
				case 0x2044: item = new Mohawk( hue ); break;
				case 0x2045: item = new PageboyHair( hue ); break;
				case 0x2046: item = new BunsHair( hue ); break;
				case 0x2047: item = new Afro( hue ); break;
				case 0x2048: item = new ReceedingHair( hue ); break;
				case 0x2049: item = new TwoPigTails( hue ); break;
				case 0x204A: item = new KrisnaHair( hue ); break;
				case 0x203B: item = new ShortHair( hue ); break;
				case 0x203C: item = new LongHair( hue ); break;
				case 0x203D: item = new PonyTail( hue ); break;
				default: return;
			}

			m.AddItem( item );
		}

		private static void AddBeard( Mobile m, int itemID, int hue )
		{
			if ( m.Female )
				return;

			Item item;

			switch ( itemID & 0x3FFF )
			{
				case 0x203E: item = new LongBeard( hue ); break;
				case 0x203F: item = new ShortBeard( hue ); break;
				case 0x2040: item = new Goatee( hue ); break;
				case 0x2041: item = new Mustache( hue ); break;
				case 0x204B: item = new MediumShortBeard( hue ); break;
				case 0x204C: item = new MediumLongBeard( hue ); break;
				case 0x204D: item = new Vandyke( hue ); break;
				default: return;
			}

			m.AddItem( item );
		}

		private static Mobile CreateMobile( Account a )
		{
			for ( int i = 0; i < 5; ++i )
				if ( a[i] == null )
					return a[i] = new Mobile();

			return null;
		}

		private static void EventSink_CharacterCreated( CharacterCreatedEventArgs args )
		{
			Mobile newChar = CreateMobile( args.Account );

			if ( newChar == null )
			{
				Console.WriteLine( "Login: {0}: Character creation failed, account full", args.State );
				return;
			}

			args.Mobile = newChar;

			newChar.Player = true;
			newChar.AccessLevel = args.Account.AccessLevel;
			newChar.Female = args.Female;
			newChar.Body = newChar.Female ? 0x191 : 0x190;
			newChar.Hue = Utility.ClipSkinHue( args.Hue & 0x3FFF ) | 0x8000;
			newChar.Hunger = 20;

			SetName( newChar, args.Name );
			SetStats( newChar, args.Str, args.Dex, args.Int );
			SetSkills( newChar, args.Skills );

			AddHair( newChar, args.HairID, Utility.ClipHairHue( args.HairHue & 0x3FFF ) );
			AddBeard( newChar, args.BeardID, Utility.ClipHairHue( args.BeardHue & 0x3FFF ) );

			AddShirt( newChar, args.ShirtHue );
			AddPants( newChar, args.PantsHue );
			AddShoes( newChar );

			AddBackpack( newChar );

			FillBankbox( newChar );

			//CityInfo city = args.City;
			CityInfo city = new CityInfo( "Britain", "Sweet Dreams Inn", 1496, 1628, 10 );

			newChar.Location = city.Location;
			newChar.Map = Map.Felucca;

			Console.WriteLine( "Login: {0}: New character being created (account={1})", args.State, args.Account.Username );
			Console.WriteLine( " - Character: {0} (serial={1})", newChar.Name, newChar.Serial );
			Console.WriteLine( " - Started: {0} {1}", city.City, city.Location );

			new WelcomeTimer( newChar ).Start();
		}

		private static void SetStats( Mobile m, int str, int dex, int intel )
		{
			if ( str < 10 || str > 60 || dex < 10 || dex > 60 || intel < 10 || intel > 60 || (str + dex + intel) != 80 )
			{
				str = 10;
				dex = 10;
				intel = 10;
			}

			m.InitStats( str, dex, intel );
		}

		private static void SetName( Mobile m, string name )
		{
			name = name.Trim();

			if ( !NameVerification.Validate( name, 2, 16, true, true, true, 1, NameVerification.SpaceDashPeriodQuote ) )
				name = "Generic Player";

			m.Name = name;
		}

		private static bool ValidSkills( SkillNameValue[] skills )
		{
			int total = 0;

			for ( int i = 0; i < skills.Length; ++i )
			{
				total += skills[i].Value;

				for ( int j = i + 1; j < skills.Length; ++j )
					if ( skills[j].Value > 0 && skills[j].Name == skills[i].Name )
						return false;
			}

			return ( total == 100 );
		}

		private static Mobile m_Mobile;
		private static Container m_Pack;

		private static void SetSkills( Mobile m, SkillNameValue[] skills )
		{
			if ( !ValidSkills( skills ) )
				return;

			m_Mobile = m;
			m_Pack = m.Backpack;

			for ( int i = 0; i < skills.Length; ++i )
			{
				SkillNameValue snv = skills[i];

				if ( snv.Value > 0 && snv.Name != SkillName.Stealth && snv.Name != SkillName.RemoveTrap )
				{
					Skill skill = m.Skills[snv.Name];

					if ( skill != null )
					{
						skill.BaseFixedPoint = snv.Value * 10;

						AddSkillItems( snv.Name );
					}
				}
			}
		}

		private static void EquipItem( Item item )
		{
			item.LootType = LootType.Newbied;

			if ( m_Mobile != null && m_Mobile.EquipItem( item ) )
				return;

			if ( m_Pack != null )
				m_Pack.DropItem( item );
			else
				item.Delete();
		}

		private static void PackItem( Item item )
		{
			item.LootType = LootType.Newbied;

			if ( m_Pack != null )
				m_Pack.DropItem( item );
			else
				item.Delete();
		}

		private static void PackInstrument()
		{
			switch ( Utility.Random( 6 ) )
			{
				case 0: PackItem( new Drums() ); break;
				case 1: PackItem( new Harp() ); break;
				case 2: PackItem( new LapHarp() ); break;
				case 3: PackItem( new Lute() ); break;
				case 4: PackItem( new Tambourine() ); break;
				case 5: PackItem( new TambourineTassel() ); break;
			}
		}

		private static void PackScroll( int circle )
		{
			switch ( Utility.Random( 8 ) * (circle * 8) )
			{
				case  0: PackItem( new ClumsyScroll() ); break;
				case  1: PackItem( new CreateFoodScroll() ); break;
				case  2: PackItem( new FeeblemindScroll() ); break;
				case  3: PackItem( new HealScroll() ); break;
				case  4: PackItem( new MagicArrowScroll() ); break;
				case  5: PackItem( new NightSightScroll() ); break;
				case  6: PackItem( new ReactiveArmorScroll() ); break;
				case  7: PackItem( new WeakenScroll() ); break;
				case  8: PackItem( new AgilityScroll() ); break;
				case  9: PackItem( new CunningScroll() ); break;
				case 10: PackItem( new CureScroll() ); break;
				case 11: PackItem( new HarmScroll() ); break;
				case 12: PackItem( new MagicTrapScroll() ); break;
				case 13: PackItem( new MagicUnTrapScroll() ); break;
				case 14: PackItem( new ProtectionScroll() ); break;
				case 15: PackItem( new StrengthScroll() ); break;
				case 16: PackItem( new BlessScroll() ); break;
				case 17: PackItem( new FireballScroll() ); break;
				case 18: PackItem( new MagicLockScroll() ); break;
				case 19: PackItem( new PoisonScroll() ); break;
				case 20: PackItem( new TelekinisisScroll() ); break;
				case 21: PackItem( new TeleportScroll() ); break;
				case 22: PackItem( new UnlockScroll() ); break;
				case 23: PackItem( new WallOfStoneScroll() ); break;
			}
		}

		private static void AddSkillItems( SkillName skill )
		{
			switch ( skill )
			{
				case SkillName.Alchemy:
				{
					PackItem( new Bottle( 4 ) );
					PackItem( new MortarPestle() );
					EquipItem( new Robe( Utility.RandomPinkHue() ) );
					break;
				}
				case SkillName.Anatomy:
				{
					PackItem( new Bandage( 3 ) );
					EquipItem( new Robe( Utility.RandomYellowHue() ) );
					break;
				}
				case SkillName.AnimalLore:
				{
					EquipItem( new ShepherdsCrook() );
					EquipItem( new Robe( Utility.RandomBlueHue() ) );
					break;
				}
				case SkillName.Archery:
				{
					PackItem( new Arrow( 25 ) );
					EquipItem( new Bow() );
					break;
				}
				// TODO: ArmsLore
				case SkillName.Begging:
				{
					EquipItem( new GnarledStaff() );
					break;
				}
				case SkillName.Blacksmith:
				{
					PackItem( new Tongs() );
					PackItem( new Pickaxe() );
					PackItem( new Pickaxe() );
					PackItem( new IronIngot( 50 ) );
					EquipItem( new HalfApron( Utility.RandomYellowHue() ) );
					break;
				}
				case SkillName.Fletching:
				{
					PackItem( new Log( 14 ) ); // TODO: Should be boards
					PackItem( new Feather( 5 ) );
					PackItem( new Shaft( 5 ) );
					break;
				}
				case SkillName.Camping:
				{
					// TODO: Bedroll
					PackItem( new Kindling( 5 ) );
					break;
				}
				case SkillName.Carpentry:
				{
					PackItem( new Log( 10 ) ); // TODO: Should be boards
					PackItem( new Saw() );
					EquipItem( new HalfApron( Utility.RandomYellowHue() ) );
					break;
				}
				case SkillName.Cartography:
				{
					// TODO: 4 blank maps
					PackItem( new Sextant() );
					break;
				}
				case SkillName.Cooking:
				{
					PackItem( new Kindling( 2 ) );
					PackItem( new RawLambLeg() );
					PackItem( new RawChickenLeg() );
					PackItem( new RawFishSteak() );
					PackItem( new SackFlour() );
					PackItem( new PitcherWater() );
					break;
				}
				case SkillName.DetectHidden:
				{
					EquipItem( new Cloak( 0x455 ) );
					break;
				}
				case SkillName.Discordance:
				{
					PackInstrument();
					break;
				}
				case SkillName.Fencing:
				{
					EquipItem( new Kryss() );
					break;
				}
				case SkillName.Fishing:
				{
					EquipItem( new FishingPole() );
					EquipItem( new FloppyHat( Utility.RandomYellowHue() ) );
					break;
				}
				case SkillName.Healing:
				{
					PackItem( new Bandage( 50 ) );
					PackItem( new Scissors() );
					break;
				}
				case SkillName.Herding:
				{
					EquipItem( new ShepherdsCrook() );
					break;
				}
				case SkillName.Hiding:
				{
					EquipItem( new Cloak( 0x455 ) );
					break;
				}
				case SkillName.Inscribe:
				{
					PackItem( new BlankScroll( 2 ) );
					// TODO: Blank book
					break;
				}
				case SkillName.ItemID:
				{
					EquipItem( new GnarledStaff() );
					break;
				}
				case SkillName.Lockpicking:
				{
					PackItem( new Lockpick( 20 ) );
					break;
				}
				case SkillName.Lumberjacking:
				{
					EquipItem( new Hatchet() );
					break;
				}
				case SkillName.Macing:
				{
					EquipItem( new Club() );
					break;
				}
				case SkillName.Magery:
				{
					BagOfReagents regs = new BagOfReagents( 30 );

					foreach ( Item item in regs.Items )
						item.LootType = LootType.Newbied;

					PackItem( regs );

					regs.LootType = LootType.Regular;

					PackScroll( 0 );
					PackScroll( 1 );
					PackScroll( 2 );

					Spellbook book = new Spellbook();
					book.Content = 0x382A8C38;

					EquipItem( book );
					break;
				}
				case SkillName.Mining:
				{
					PackItem( new Pickaxe() );
					break;
				}
				case SkillName.Musicianship:
				{
					PackInstrument();
					break;
				}
				// TODO: Parry
				case SkillName.Peacemaking:
				{
					PackInstrument();
					break;
				}
				case SkillName.Poisoning:
				{
					PackItem( new LesserPoisonPotion() );
					PackItem( new LesserPoisonPotion() );
					break;
				}
				case SkillName.Provocation:
				{
					PackInstrument();
					break;
				}
				case SkillName.Snooping:
				{
					PackItem( new Lockpick( 20 ) );
					break;
				}
				case SkillName.SpiritSpeak:
				{
					EquipItem( new Cloak( 0x455 ) );
					break;
				}
				case SkillName.Stealing:
				{
					PackItem( new Lockpick( 20 ) );
					break;
				}
				case SkillName.Swords:
				{
					EquipItem( new Katana() );
					break;
				}
				case SkillName.Tactics:
				{
					EquipItem( new Katana() );
					break;
				}
				case SkillName.Tailoring:
				{
					PackItem( new BoltOfCloth() );
					PackItem( new SewingKit() );
					break;
				}
				case SkillName.Tracking:
				{
					if ( m_Mobile != null )
					{
						Item shoes = m_Mobile.FindItemOnLayer( Layer.Shoes );

						if ( shoes != null )
							shoes.Delete();
					}

					EquipItem( new Boots( Utility.RandomYellowHue() ) );
					EquipItem( new SkinningKnife() );
					break;
				}
				case SkillName.Veterinary:
				{
					PackItem( new Bandage( 5 ) );
					PackItem( new Scissors() );
					break;
				}
				case SkillName.Wrestling:
				{
					EquipItem( new LeatherGloves() );
					break;
				}
			}
		}
	}
}