using System;
using System.Collections;
using Server.Items;

namespace Server.Mobiles
{
	public class GenericSellInfo : IShopSellInfo
	{
		private Hashtable m_Table = new Hashtable();
		private ArrayList m_MyTypes = new ArrayList();

		public GenericSellInfo()
		{
		}

		public void Add(Type type, int price)
		{
			m_Table[type] = price;
			m_MyTypes.Add( type );
		}

		public int GetSellPriceFor( Item item )
		{
			int price = (int)m_Table[item.GetType()];

			if ( item is BaseArmor )
			{
				BaseArmor armor = (BaseArmor)item;

				if ( armor.Quality == ArmorQuality.Low )
					price = (int)( price * 0.60 );
				else if ( armor.Quality == ArmorQuality.Exceptional )
					price = (int)( price * 1.25 );

				price += 100 * (int)armor.Durability;

				price += 100 * (int)armor.ProtectionLevel;

				if ( price < 1 )
					price = 1;
			}

			else if ( item is BaseWeapon )
			{
				BaseWeapon weapon = (BaseWeapon)item;

				if ( weapon.Quality == WeaponQuality.Low )
					price = (int)( price * 0.60 );
				else if ( weapon.Quality == WeaponQuality.Exceptional )
					price = (int)( price * 1.25 );

				price += 100 * (int)weapon.DurabilityLevel;

				price += 100 * (int)weapon.DamageLevel;

				if ( price < 1 )
					price = 1;
			}

			return price;
		}

		public int GetBuyPriceFor( Item item )
		{
			return (int)( 1.90 * GetSellPriceFor( item ) );
		}

		public Type[] Types
		{
			get
			{
				return (Type[])m_MyTypes.ToArray( typeof( Type ) );
			}
		}

		public string GetNameFor( Item item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return (1020000 + (item.ItemID & 0x3FFF)).ToString();
		}

		public bool IsSellable( Item item )
		{
			if ( item.Hue != 0 )
				return false;

			return IsInList( item.GetType() );
		}
	 
		public bool IsResellable( Item item )
		{
			if ( item.Hue != 0 )
				return false;

			return IsInList( item.GetType() );
		}

		public bool IsInList( Type type )
		{
			Object o = m_Table[type];

			if ( o == null )
				return false;
			else
				return true;
		}
	}
}
