using System;
using Server.Targeting;
using Server.Items;
using Server.Network;

namespace Server.SkillHandlers
{
	public class Stealing
	{
		public static void Initialize()
		{
			SkillInfo.Table[33].Callback = new SkillUseCallback( OnUse );
		}

		private class StealingTarget : Target
		{
			private Mobile m_Thief;

			public StealingTarget( Mobile thief ) : base ( 1, false, TargetFlags.None )
			{
				m_Thief = thief;

				AllowNonlocal = true;
			}

			private Item TryStealItem( Item toSteal )
			{
				Item stolen = null;

				object root = toSteal.RootParent;

				if ( !m_Thief.CanSee( toSteal ) )
				{
					m_Thief.SendLocalizedMessage( 500237 ); // Target can not be seen.
				}
				else if ( !toSteal.Movable || toSteal.LootType == LootType.Newbied || toSteal.LootType == LootType.Blessed )
				{
					m_Thief.SendLocalizedMessage( 502710 ); // You can't steal that!
				}
				else if ( !m_Thief.InRange( toSteal.GetWorldLocation(), 1 ) )
				{
					m_Thief.SendLocalizedMessage( 502703 ); // You must be standing next to an item to steal it.
				}
				else if ( toSteal.Parent is Mobile )
				{
					m_Thief.SendLocalizedMessage( 1005585 ); // You cannot steal items which are equiped.
				}
				else if ( root == m_Thief )
				{
					m_Thief.SendLocalizedMessage( 502704 ); // You catch yourself red-handed.
				}
				else if ( root is Mobile && ((Mobile)root).AccessLevel > AccessLevel.Player )
				{
					m_Thief.SendLocalizedMessage( 502710 ); // You can't steal that!
				}
				else if ( root is Mobile && !m_Thief.CanBeHarmful( (Mobile)root ) )
				{
				}
				else
				{
					if ( toSteal.Weight > 10.0 )
					{
						m_Thief.SendMessage( "That is too heavy to steal." );
					}
					else
					{
						if ( toSteal.Stackable && toSteal.Amount > 1 )
						{
							int amount = Utility.Random( 1, (int)((m_Thief.Skills[SkillName.Stealing].Value / 10.0) / toSteal.Weight) );

							if ( amount >= toSteal.Amount )
							{
								if ( m_Thief.CheckSkill( SkillName.Stealing, 0, toSteal.Weight*toSteal.Amount*10 ) )
									stolen = toSteal;
							}
							else
							{
								if ( m_Thief.CheckSkill( SkillName.Stealing, 0, toSteal.PileWeight*10 ) )
								{
									stolen = toSteal.Dupe( amount );
									toSteal.Amount -= amount;
								}
							}
						}
						else if ( toSteal.TotalWeight > 10 )
						{
							m_Thief.SendMessage( "That is too heavy to steal." );
							return stolen; // null
						}
						else
						{
							if ( m_Thief.CheckSkill( SkillName.Stealing, 0, toSteal.Weight*10 ) )
								stolen = toSteal;
						}

						if ( stolen != null )
							m_Thief.SendLocalizedMessage( 502724 ); // You succesfully steal the item.
						else
							m_Thief.SendLocalizedMessage( 502723 ); // You fail to steal the item.

						if ( m_Thief.Skills[SkillName.Stealing].Value < Utility.Random( 150 ) )
						{
							// 1 in 3 chance of being flagged criminal @ 100 skill

							if ( root == null )
							{
								m_Thief.CriminalAction( false );
							}
							else if ( root is Corpse && ((Corpse)root).IsCriminalAction( m_Thief ) )
							{
								m_Thief.CriminalAction( false );
							}
							else if ( root is Mobile )
							{
								Mobile mobRoot = (Mobile)root;

								m_Thief.DoHarmful( mobRoot );

								string message = String.Format( "You notice {0} trying to steal from {1}.", m_Thief.Name, mobRoot.Name );

								IPooledEnumerable eable = m_Thief.Map.GetClientsInRange( m_Thief.Location, 8 );

								foreach ( NetState ns in eable )
									if ( ns != m_Thief.NetState )
										ns.Mobile.SendMessage( message );

								eable.Free();
							}
						}
						else if ( root is Corpse && ((Corpse)root).IsCriminalAction( m_Thief ) )
						{
							m_Thief.CriminalAction( false );
						}
					}
				}

				return stolen;
			}

			protected override void OnTarget( Mobile from, object target )
			{
				from.RevealingAction();

				Item stolen = null;

				if ( target is Item )
				{
					stolen = TryStealItem( (Item)target );
				} 
				else if ( target is Mobile )
				{
					Container pack = ((Mobile)target).Backpack;

					if ( pack != null && pack.Items.Count > 0 )
					{
						int randomIndex = Utility.Random( pack.Items.Count );

						stolen = TryStealItem( (Item) pack.Items[randomIndex] );
					}
				} 
				else 
				{
					m_Thief.SendLocalizedMessage( 502710 ); // You can't steal that!
				}

				if ( stolen != null )
					from.AddToBackpack( stolen );
			}
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.Target = new Stealing.StealingTarget( m );
			m.RevealingAction();

			m.SendLocalizedMessage( 502698 ); // Which item do you want to steal?

			return TimeSpan.FromSeconds( 10.0 );
		}
	}
}