using System;
using System.Collections;
using Server;
using Server.Gumps;
using Server.Network;

namespace Server.SkillHandlers
{
	public class Tracking
	{
		public static void Initialize()
		{
			SkillInfo.Table[(int)SkillName.Tracking].Callback = new SkillUseCallback( OnUse );
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.SendLocalizedMessage( 1011350 ); // What do you wish to track?

			m.CloseGump( typeof( TrackWhatGump ) );
			m.CloseGump( typeof( TrackWhoGump ) );
			m.SendGump( new TrackWhatGump( m ) );

			return TimeSpan.FromSeconds( 10.0 ); // 10 second delay before beign able to re-use a skill
		}
	}

	public class TrackWhatGump : Gump
	{
		private Mobile m_From;
		private bool m_Success;

		public TrackWhatGump( Mobile from ) : base( 20, 30 )
		{
			m_From = from;
			m_Success = from.CheckSkill( SkillName.Tracking, 0.0, 100.0 );

			AddPage( 0 );

			AddBackground( 0, 0, 440, 135, 5054 );

			AddBackground( 10, 10, 420, 75, 2620 );
			AddBackground( 10, 85, 420, 25, 3000 );

			AddItem( 20, 20, 9682 );
			AddButton( 20, 110, 4005, 4007, 1, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 20, 90, 100, 20, 1018087, false, false ); // Animals

			AddItem( 120, 20, 9607 );
			AddButton( 120, 110, 4005, 4007, 2, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 120, 90, 100, 20, 1018088, false, false ); // Monsters

			AddItem( 220, 20, 8454 );
			AddButton( 220, 110, 4005, 4007, 3, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 220, 90, 100, 20, 1018089, false, false ); // Human NPCs

			AddItem( 320, 20, 8455 );
			AddButton( 320, 110, 4005, 4007, 4, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 320, 90, 100, 20, 1018090, false, false ); // Players
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			if ( info.ButtonID >= 1 && info.ButtonID <= 4 )
				TrackWhoGump.DisplayTo( m_Success, m_From, info.ButtonID - 1 );
		}
	}

	public delegate bool TrackTypeDelegate( Mobile m );

	public class TrackWhoGump : Gump
	{
		private Mobile m_From;
		private int m_Range;

		private static TrackTypeDelegate[] m_Delegates = new TrackTypeDelegate[]
			{
				new TrackTypeDelegate( IsAnimal ),
				new TrackTypeDelegate( IsMonster ),
				new TrackTypeDelegate( IsHumanNPC ),
				new TrackTypeDelegate( IsPlayer )
			};

		private class InternalSorter : IComparer
		{
			private Mobile m_From;

			public InternalSorter( Mobile from )
			{
				m_From = from;
			}

			public int Compare( object x, object y )
			{
				if ( x == null && y == null )
					return 0;
				else if ( x == null )
					return -1;
				else if ( y == null )
					return 1;

				Mobile a = x as Mobile;
				Mobile b = y as Mobile;

				if ( a == null || b == null )
					throw new ArgumentException();

				return m_From.GetDistanceToSqrt( a ).CompareTo( m_From.GetDistanceToSqrt( b ) );
			}
		}

		public static void DisplayTo( bool success, Mobile from, int type )
		{
			if ( !success )
			{
				from.SendLocalizedMessage( 1018092 ); // You see no evidence of those in the area.
				return;
			}

			Map map = from.Map;

			if ( map == null )
				return;

			TrackTypeDelegate check = m_Delegates[type];

			int range = (int)(from.Skills[SkillName.Tracking].Value / 5);

			ArrayList list = new ArrayList();
			IPooledEnumerable eable = map.GetMobilesInRange( from.Location, range );

			foreach ( Mobile m in eable )
			{
				if ( m != from && check( m ) )
					list.Add( m );
			}

			eable.Free();

			if ( list.Count > 0 )
			{
				list.Sort( new InternalSorter( from ) );

				from.SendGump( new TrackWhoGump( from, list, range ) );
				from.SendLocalizedMessage( 1018093 ); // Select the one you would like to track.
			}
			else
			{
				if ( type == 0 )
					from.SendLocalizedMessage( 502991 ); // You see no evidence of animals in the area.
				else if ( type == 1 )
					from.SendLocalizedMessage( 502993 ); // You see no evidence of creatures in the area.
				else
					from.SendLocalizedMessage( 502995 ); // You see no evidence of people in the area.
			}
		}

		private static bool IsAnimal( Mobile m )
		{
			return ( !m.Player && m.Body.IsAnimal );
		}

		private static bool IsMonster( Mobile m )
		{
			return ( !m.Player && m.Body.IsMonster );
		}

		private static bool IsHumanNPC( Mobile m )
		{
			return ( !m.Player && m.Body.IsHuman );
		}

		private static bool IsPlayer( Mobile m )
		{
			return m.Player;
		}

		private static int GetItemID( Mobile m )
		{
			int val = m.Body.BodyID;

			switch ( val )
			{
				case 848: return 0x259E;
				case 846: return 0x2599;
				case 843: return 0x2123;
				case 838: return 0x25C1;
				case 837: return 0x25C0;
				case 835: return 0x2125;
				case 834: return 0x2597;
				case 833: return 0x211A;
				case 832: return 0x211A;
				case 831: return 0x211A;
				case 826: return 0x259A;
				case 792: return 0x2609;
				case 786: return 0x260A;
				case 784: return 0x2605;
				case 780: return 0x2608;
				case 779: return 0x2607;
				case 777: return 0x260D;
				case 774: return 0x25FB;
				case 763: return 0x260B;
				case 757: return 0x260B;
				case 756: return 0x260C;
				case 400: return 0x2106;
				case 401: return 0x2107;
				case 291: return 0x2126;
				case 238: return 0x2123;
				case 237: return 0x20D4;
				case 234: return 0x20D4;
				case 233: return 0x20F0;
				case 232: return 0x20EF;
				case 231: return 0x2103;
				case 228: return 0x259E;
				case 226: return 0x2599;
				case 225: return 0x25D3;
				case 217: return 0x2588;
				case 216: return 0x20EF;
				case 213: return 0x20E1;
				case 212: return 0x211E;
				case 211: return 0x2118;
				case 208: return 0x20D1;
				case 205: return 0x2125;
				case 204: return 0x259A;
				case 201: return 0x211B;
				case 200: return 0x2598;
				case 193: return 0x2615;
				case 192: return 0x25CE;
				case 190: return 0x21F1;
				case 188: return 0x2615;
				case 187: return 0x2615;
				case 180: return 0x20D6;
				case 179: return 0x259C;
				case 178: return 0x259C;
				case 177: return 0x259C;
				case 173: return 0x25C3;
				case 172: return 0x20D6;
				case 167: return 0x2118;
				case 166: return 0x20D7;
				case 165: return 0x2100;
				case 164: return 0x20ED;
				case 163: return 0x25DC;
				case 162: return 0x25DB;
				case 161: return 0x25DA;
				case 160: return 0x25D8;
				case 159: return 0x25D8;
				case 158: return 0x25D6;
				case 157: return 0x25C3;
				case 151: return 0x20F1;
				case 122: return 0x25CE;
				case 121: return 0x259E;
				case 120: return 0x259E;
				case 119: return 0x259E;
				case 118: return 0x259E;
				case 117: return 0x259D;
				case 116: return 0x259C;
				case 115: return 0x259B;
				case 114: return 0x259B;
				case 113: return 0x20D7;
				case 112: return 0x20D7;
				case 111: return 0x20D7;
				case 110: return 0x20D7;
				case 109: return 0x20D7;
				case 108: return 0x20D7;
				case 107: return 0x20D7;
				case 106: return 0x20D6;
				case 105: return 0x20D6;
				case 104: return 0x20D6;
				case 103: return 0x20D6;
				case 102: return 0x2584;
				case 101: return 0x2581;
				case 100: return 0x25D2;
				case  99: return 0x25D1;
				case  98: return 0x2596;
				case  97: return 0x2596;
				case  81: return 0x2130;
				case  75: return 0x212D;
				case  62: return 0x20D6;
				case  61: return 0x20D6;
				case  60: return 0x20D6;
				case  59: return 0x20D6;
				case  58: return 0x2100;
				case  52: return 0x25C2;
				case  49: return 0x20D6;
				case  46: return 0x20D6;
				case  43: return 0x2587;
				case  40: return 0x2586;
				case  38: return 0x2585;
				case  37: return 0x25D2;
				case  34: return 0x25D2;
				case  28: return 0x25C6;
				case  27: return 0x25D1;
				case  25: return 0x25D1;
				case  23: return 0x25D0;
				case  20: return 0x25C5;
				case  19: return 0x25C4;
				case  16: return 0x25DD;
				case  15: return 0x25D9;
				case  14: return 0x20D7;
				case  13: return 0x25D7;
				case  12: return 0x20D6;
				case  11: return 0x25C4;
				case  10: return 0x2584;
				case   9: return 0x2584;
				case   6: return 0x211A;
				case   5: return 0x211D;

				// TODO: All of these
#if false
123	0	0	FFFFFFFF	0	1.3	1.3	1.3	ethereal_warriors_male
175	0	0	FFFFFFFF	0	2.3	2.3	2.3	ethereal_warriors_male
2	0	0	FFFFFFFF	0	1.3	1.3	1.3	ettins_ettin
18	0	0	FFFFFFFF	0	1.3	1.3	1.3	ettins_ettin
63	2	214	FFFFFFFF	0	1.3	1.3	1.3	felines_cougar
64	2	214	FFFFFFFF	0	1.3	1.3	1.3	felines_leopard_snow
65	2	214	FFFFFFFF	0	1.3	1.3	1.3	felines_leopard_snow
214	2	0	FFFFFFFF	0	1.3	1.3	1.3	felines_panther
127	2	214	FFFFFFFF	0	1.3	1.3	1.3	felines_predator_hellcat
128	0	0	FFFFFFFF	0	1.3	1.3	1.3	fey_race_pixie
176	0	0	FFFFFFFF	0	2.3	2.3	2.3	fey_race_pixie
781	0	0	FFFFFFFF	0	1.3	1.3	1.3	fire_ant_worker
782	0	0	FFFFFFFF	0	1.3	1.3	1.3	fire_ant_warrior
783	0	0	FFFFFFFF	0	1.3	1.3	1.3	fire_ant_queen
804	0	783	FFFFFFFF	0	1.5	1.5	1.5	fire_ant_matriarch
805	0	781	FFFFFFFF	0	1.3	1.3	1.3	black_ant_worker
806	0	782	FFFFFFFF	0	1.3	1.3	1.3	black_ant_warrior
807	0	783	FFFFFFFF	0	1.3	1.3	1.3	black_ant_queen
808	0	783	FFFFFFFF	0	1.5	1.5	1.5	black_ant_matriarch
8	0	0	FFFFFFFF	0	1.3	1.3	1.3	flails_corpser
47	0	0	FFFFFFFF	0	1.3	1.3	1.3	flails_reaper
66	0	0	FFFFFFFF	0	1.3	1.3	1.3	flails_swamp_tentacles
129	0	0	FFFFFFFF	0	1.3	1.3	1.3	flails_swamp_tentacles
80	0	0	FFFFFFFF	0	1.3	1.3	1.3	frogs_frog_giant
785	0	0	FFFFFFFF	0	1.3	1.3	1.3	four_armed_demon
4	0	0	FFFFFFFF	0	1.3	1.3	1.3	gargoyles_gargoyle
130	0	4	FFFFFFFF	0	1.3	1.3	1.3	gargoyles_gargoyle_blistering
67	0	4	FFFFFFFF	0	1.3	1.3	1.3	gargoyles_gargoyle_stone
753	0	4	FFFFFFFF	0	1.5	1.5	1.5	gargoyles_gargoyle_slave
754	0	4	FFFFFFFF	0	1.7	1.7	1.7	gargoyles_gargoyle_enforcer
755	0	4	FFFFFFFF	0	2.3	2.3	2.3	gargoyles_gargoyle_guard
758	0	4	FFFFFFFF	0	1.5	1.5	1.5	gargoyles_gargoyle_shopkeeper
22	0	0	FFFFFFFF	0	1.3	1.3	1.3	gazers_gazer
68	0	22	FFFFFFFF	0	1.3	1.3	1.3	gazers_gazer_elder
69	0	22	FFFFFFFF	0	1.3	1.3	1.3	gazers_gazer_elder
131	0	0	FFFFFFFF	0	1.3	1.3	1.3	genies_efreet
791	2	0	FFFFFFFF	0	1.42	1.42	1.42	giant_beetle
195	2	791	FFFFFFFF	0	1.42	1.42	1.42	giant_beetle_ethereal
752	0	14	FFFFFFFF	0	1.9	1.9	1.9	golem_iron
29	2	0	FFFFFFFF	0	1.3	1.3	1.3	gorillas_silverback
29	2	0	FFFFFFFF	0	1.3	1.3	1.3	gorillas_silverback
30	0	0	FFFFFFFF	0	1.3	1.3	1.3	harpies_harpy
73	0	30	FFFFFFFF	0	1.3	1.3	1.3	harpies_harpy_stone
31	0	0	FFFFFFFF	0	1.3	1.3	1.3	headless_headless
776	0	0	FFFFFFFF	0	1.3	1.3	1.3	horde_demon
795	0	776	FFFFFFFF	0	2.3	2.3	3.3	horde_demon
796	0	776	FFFFFFFF	0	3.3	3.3	3.3	horde_demon
74	0	0	FFFFFFFF	0	1.3	1.3	1.3	imps_imp
168	0	74	FFFFFFFF	0	1.3	1.3	1.3	imps_shadow_fiend
764	0	400	FFFFFFFF	0	1.55	1.45	1.55	Juka_Warrior
765 	0	401  	FFFFFFFF	0	1.55	1.4	1.55	Juka_Mage
768 	0	0       FFFFFFFF	0	1.55	1.45	1.55	Juggernaut
132	2	998	FFFFFFFF	0	1.5	1.5	1.5	kirin_kirin
191	2	998	FFFFFFFF	0	1.5	1.5	1.5	kirin_kirin_ethereal
998	0	0	FFFFFFFF	0	1.5	1.5	1.5	kirin_kirin
77	0	0	FFFFFFFF	0	1.3	1.3	1.3	krakens_kraken
24	0	0	FFFFFFFF	0	1.5	1.5	1.5	liche_liche_retro
78	0	24	FFFFFFFF	0	1.5	1.5	1.5	liche_liche_lord_retro
79	0	24	FFFFFFFF	0	1.5	1.5	1.5	liche_liche_lord_retro
82	0	24	FFFFFFFF	0	1.5	1.5	1.5	liche_liche_lord_retro
33	0	0	FFFFFFFF	0	1.3	1.3	1.3	lizard_race_lizardman
35	0	0	FFFFFFFF	0	1.3	1.3	1.3	lizard_race_lizardman
36	0	0	FFFFFFFF	0	1.3	1.3	1.3	lizard_race_lizardman
202	2	0	FFFFFFFF	0	1.3	1.3	1.3	lizards_alligator_brown
202	2	0	FFFFFFFF	0	1.3	1.3	1.3	lizards_alligator_brown
133	2	0	FFFFFFFF	0	1.3	1.3	1.3	lizards_alligator_small
134	2	0	FFFFFFFF	0	1.3	1.3	1.3	lizards_komodo_dragon
206	0	0	FFFFFFFF	0	1.3	1.3	1.3	lizards_lava_lizard
220	2	0	FFFFFFFF	0	1.5	1.5	1.5	llamas_llama
828	2	0	FFFFFFFF	0	1.5	1.5	1.5	llamas_llama
170	2	220	FFFFFFFF	0	1.5	1.5	1.5	llamas_llama_ethereal
292	2	220	FFFFFFFF	0	1.5	1.5	1.5	llamas_llama_pack
124	0	0	FFFFFFFF	0	1.3	1.3	1.3	mages_evil_mage
125	0	124	FFFFFFFF	0	1.3	1.3	1.3	mages_evil_mage_master
126	0	124	FFFFFFFF	0	1.3	1.3	1.3	mages_evil_mage_master
770 	0	401	FFFFFFFF 	0	1.55	1.4	1.55	Meer_Mage
771     0	400	FFFFFFFF	0	1.55	1.45	1.55	Meer_Warrior
797	0	0	FFFFFFFF	0	1.3	1.3	1.3	mf_dragon_fire
798	0	797	FFFFFFFF	0	1.3	1.3	1.3	mf_dragon_rust
39	0	0	FFFFFFFF	0	0.5	0.5	0.5	mongbats_mongbat
766 	0	400 	FFFFFFFF	0	1.55	1.45	1.55	NPC_Kabur
767 	0	769	FFFFFFFF	0	1.55	1.45	1.55	NPC_Blackthorn_Cohort
769 	0	0 	FFFFFFFF	0	1.55	1.45	1.55	NPC_Blackthorn
772 	0	0	FFFFFFFF	0	1.55	1.45	1.55	NPC_Adranath
773 	0	401	FFFFFFFF	0	1.55	1.55	1.55	NPC_Cpt_Dasha
1	0	0	FFFFFFFF	0	1.3	1.3	1.3	ogres_ogre
83	0	0	FFFFFFFF	0	1.3	1.3	1.3	ogres_ogre_lord
84	0	0	FFFFFFFF	0	1.3	1.3	1.3	ogres_ogre_lord
135	0	83	FFFFFFFF	0	1.3	1.3	1.3	ogres_ogre_lord_arctic
136	0	0	FFFFFFFF	0	1.3	1.3	1.3	ophidians_ophidian_archmage
137	0	0	FFFFFFFF	0	1.3	1.3	1.3	ophidians_ophidian_knight
85	0	0	FFFFFFFF	0	1.3	1.3	1.3	ophidians_ophidian_mage
87	0	0	FFFFFFFF	0	1.3	1.3	1.3	ophidians_ophidian_queen
86	0	0	FFFFFFFF	0	1.3	1.3	1.3	ophidians_ophidian_warrior
17	0	0	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc
41	0	0	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc
7	0	0	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc_captain
138	0	0	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc_lord
139	0	0	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc_lord
140	0	0	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc_shaman
181	0	140	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc_scout
182	0	17	FFFFFFFF	0	1.3	1.3	1.3	orcs_orc_bomber
189	0	138	FFFFFFFF	0	3.5	3.0	3.5	orcs_orc_lord
210	2	0	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_desert
825	2	0	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_desert
218	2	210	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_forest
826	2	210	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_forest
219	2	210	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_frenzied
827	2	210	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_frenzied
171	2	210	FFFFFFFF	0	1.5	1.5	1.5	ostards_ostard_ethereal
141	0	0	FFFFFFFF	0	1.3	1.3	1.3	paladins_paladin
775	0	0	FFFFFFFF	0	1.3	1.3	1.3	plague_beast
290	2	0	FFFFFFFF	0	1.3	1.3	1.3	porcines_boar
203	2	0	FFFFFFFF	0	1.3	1.3	1.3	porcines_pig
789	0	0	FFFFFFFF	0	1.3	1.3	1.3	quagmire
42	0	0	FFFFFFFF	0	1.3	1.3	1.3	rat_race_ratman
44	0	0	FFFFFFFF	0	1.3	1.3	1.3	rat_race_ratman
45	0	0	FFFFFFFF	0	1.3	1.3	1.3	rat_race_ratman
142	0	0	FFFFFFFF	0	1.3	1.3	1.3	rat_race_ratman
143	0	42	FFFFFFFF	0	1.3	1.3	1.3	rat_race_shaman
215	0	0	FFFFFFFF	0	1.3	1.3	1.3	rodents_rat_giant
839	2	0	FFFFFFFF	0	1.3	1.3	1.3	ridgeback_ridgeback
840	2	839	FFFFFFFF	0	1.3	1.3	1.3	ridgeback_giant
841	2	839	FFFFFFFF	0	1.3	1.3	1.3	ridgeback_flame
842	2	839	FFFFFFFF	0	1.3	1.3	1.3	ridgeback_hatchling
209	2	0	FFFFFFFF	0	1.3	1.3	1.3	ruminants_goat_billy
88	2	209	FFFFFFFF	0	1.3	1.3	1.3	ruminants_goat_mountain
207	2	0	FFFFFFFF	0	1.3	1.3	1.3	ruminants_sheep
223	2	207	FFFFFFFF	0	1.3	1.3	1.3	ruminants_sheep_shorn
790	0	0	FFFFFFFF	0	1.3	1.3	1.3	sand_vortex
48	0	0	FFFFFFFF	0	1	1	1	scorpions_giant
144	1	0	FFFFFFFF	0	1.5	1.5	1.5	sea_horse_sea_horse
150	1	0	FFFFFFFF	0	0.8	0.8	0.8	sea_serpents_sea_serpent
145	1	0	FFFFFFFF	0	1	1	1	sea_serpents_sea_serpent
21	0	0	FFFFFFFF	0	1.3	1.3	1.3	serpents_snake_giant
89	0	21	FFFFFFFF	0	1.3	1.3	1.3	serpents_snake_giant_ice
90	0	21	FFFFFFFF	0	1.3	1.3	1.3	serpents_snake_giant_lava
91	0	21	FFFFFFFF	0	1.3	1.3	1.3	serpents_snake_giant_silver
92	0	21	FFFFFFFF	0	1.3	1.3	1.3	serpents_snake_giant_silver
93	0	21	FFFFFFFF	0	1.3	1.3	1.3	serpents_snake_giant_silver
146	0	0	FFFFFFFF	0	1.3	1.3	1.3	shadowlords_shadowlord
793	2	0	FFFFFFFF	0	1.3	1.3	1.3	skeletal_mount
50	0	0	FFFFFFFF	0	1.5	1.5	1.5	skeletons_skeleton
56	0	0	FFFFFFFF	0	1.5	1.5	1.5	skeletons_skeleton
57	0	0	FFFFFFFF	0	1.5	1.5	1.5	skeletons_skeleton
147	0	0	FFFFFFFF	0	1.5	1.5	1.5	skeletons_skeleton_knight
148	0	0	FFFFFFFF	0	1.5	1.5	1.5	skeletons_skeleton_mage
51	0	0	FFFFFFFF	0	1.3	1.3	1.3	slimes_slime
94	0	51	FFFFFFFF	0	1.3	1.3	1.3	slimes_slime_frost
96	0	51	FFFFFFFF	0	1.3	1.3	1.3	slimes_slime_frost
788	0	0	FFFFFFFF	0	1.3	1.3	1.3	sphinx
574	0	0	FFFFFFFF	0	1.3	1.3	1.3	spirits_blade_spirit
26	0	0	FFFFFFFF	0	1.3	1.3	1.3	spirits_ghost
149	0	0	FFFFFFFF	0	1.3	1.3	1.3	succubi_succubus
174	0	0	FFFFFFFF	0	2.3	2.3	2.3	succubi_succubus
794	2	0	FFFFFFFF	0	1.3	1.3	1.3	swamp_dragon
194	2	794	FFFFFFFF	0	1.3	1.3	1.3	swamp_dragon_ethereal
799	2	794	FFFFFFFF	0	1.3	1.3	1.3	swamp_dragon_armor
778	0	0	FFFFFFFF	0	1.3	1.3	1.3	swarm
152	0	0	FFFFFFFF	0	1.7	1.7	1.7	terathan_terathan_avenger
71	0	0	FFFFFFFF	0	1.7	1.7	1.7	terathan_terathan_drone
72	0	0	FFFFFFFF	0	1.7	1.7	1.7	terathan_terathan_queen
70	0	0	FFFFFFFF	0	1.7	1.7	1.7	terathan_terathan_warrior
76	0	0	FFFFFFFF	0	1.3	1.3	1.3	titans_titan
53	0	0	FFFFFFFF	0	1.3	1.3	1.3	trolls_troll
54	0	53	FFFFFFFF	0	1.3	1.3	1.3	trolls_troll
55	0	53	FFFFFFFF	0	1.3	1.3	1.3	trolls_troll_frost
153	0	0	FFFFFFFF	0	1.5	1.5	1.5	walking_dead_ghoul
154	0	0	FFFFFFFF	0	1.5	1.5	1.5	walking_dead_mummy
155	0	0	FFFFFFFF	0	1.5	1.5	1.5	walking_dead_rotting_corpse
3	0	0	FFFFFFFF	0	1.5	1.5	1.5	walking_dead_zombie
221	2	0	FFFFFFFF	0	1.3	1.3	1.3	walrus_walrus_male
#endif


				default:  return 0x1870;
			}
		}

		private ArrayList m_List;

		private TrackWhoGump( Mobile from, ArrayList list, int range ) : base( 20, 30 )
		{
			m_From = from;
			m_List = list;
			m_Range = range;

			AddPage( 0 );

			AddBackground( 0, 0, 440, 155, 5054 );

			AddBackground( 10, 10, 420, 75, 2620 );
			AddBackground( 10, 85, 420, 45, 3000 );

			if ( list.Count > 4 )
			{
				AddBackground( 0, 155, 440, 155, 5054 );

				AddBackground( 10, 165, 420, 75, 2620 );
				AddBackground( 10, 240, 420, 45, 3000 );

				if ( list.Count > 8 )
				{
					AddBackground( 0, 310, 440, 155, 5054 );

					AddBackground( 10, 320, 420, 75, 2620 );
					AddBackground( 10, 395, 420, 45, 3000 );
				}
			}

			for ( int i = 0; i < list.Count && i < 12; ++i )
			{
				Mobile m = (Mobile)list[i];

				AddItem( 20 + ((i % 4) * 100), 20 + ((i / 4) * 155), GetItemID( m ) );
				AddButton( 20 + ((i % 4) * 100), 130 + ((i / 4) * 155), 4005, 4007, i + 1, GumpButtonType.Reply, 0 );

				if ( m.Name != null )
					AddHtml( 20 + ((i % 4) * 100), 90 + ((i / 4) * 155), 90, 40, m.Name, false, false );
			}
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			int index = info.ButtonID - 1;

			if ( index >= 0 && index < m_List.Count && index < 12 )
			{
				Mobile m = (Mobile)m_List[index];

				m_From.QuestArrow = new TrackArrow( m_From, m, m_Range * 2 );
			}
		}
	}

	public class TrackArrow : QuestArrow
	{
		private Mobile m_From;
		private Timer m_Timer;

		public TrackArrow( Mobile from, Mobile target, int range ) : base( from )
		{
			m_From = from;
			m_Timer = new TrackTimer( from, target, range, this );
			m_Timer.Start();
		}

		public override void OnClick( bool rightClick )
		{
			if ( rightClick )
			{
				m_From = null;

				Stop();
			}
		}

		public override void OnStop()
		{
			m_Timer.Stop();

			if ( m_From != null )
				m_From.SendLocalizedMessage( 503177 ); // You have lost your quarry.
		}
	}

	public class TrackTimer : Timer
	{
		private Mobile m_From, m_Target;
		private int m_Range;
		private int m_LastX, m_LastY;
		private QuestArrow m_Arrow;

		public TrackTimer( Mobile from, Mobile target, int range, QuestArrow arrow ) : base( TimeSpan.FromSeconds( 0.25 ), TimeSpan.FromSeconds( 2.5 ) )
		{
			m_From = from;
			m_Target = target;
			m_Range = range;

			m_Arrow = arrow;
		}

		protected override void OnTick()
		{
			if ( !m_Arrow.Running )
			{
				Stop();
				return;
			}
			else if ( m_From.NetState == null || m_From.Deleted || m_Target.Deleted || m_From.Map != m_Target.Map || !m_From.InRange( m_Target, m_Range ) )
			{
				m_From.Send( new CancelArrow() );
				m_From.SendLocalizedMessage( 503177 ); // You have lost your quarry.

				Stop();
				return;
			}

			if ( m_LastX != m_Target.X || m_LastY != m_Target.Y )
			{
				m_LastX = m_Target.X;
				m_LastY = m_Target.Y;

				m_Arrow.Update( m_LastX, m_LastY );
			}
		}
	}
}