using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Sixth
{
	public class ExplosionSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Explosion", "Vas Ort Flam",
				SpellCircle.Sixth,
				230,
				Reagent.Bloodmoss,
				Reagent.MandrakeRoot
			);

		public ExplosionSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckHSequence( m ) )
			{
				SpellHelper.Turn( Caster, m );

				SpellHelper.CheckReflect( (int)this.Circle, Caster, ref m );

				InternalTimer t = new InternalTimer( this, m );
				t.Start();
			}

			FinishSequence();
		}

		private class InternalTimer : Timer
		{
			private Spell m_Spell;
			private Mobile m_Target;

			public InternalTimer( Spell spell, Mobile target ) : base( TimeSpan.FromSeconds( 2.5 ) )
			{
				m_Spell = spell;
				m_Target = target;
			}

			protected override void OnTick()
			{
				double damage = Utility.Random( 23, 22 );

				if ( m_Spell.CheckResisted( m_Target ) )
				{
					damage *= 0.75;

					m_Target.SendLocalizedMessage( 501783 ); // You feel yourself resisting magical energy.
				}

				damage *= m_Spell.GetDamageScalar( m_Target );

				m_Target.PlaySound( 0x307 );
				Effects.SendTargetEffect( m_Target, 0x36BD, 20 );

				SpellHelper.Damage( m_Spell, m_Target, damage );
			}
		}

		private class InternalTarget : Target
		{
			private ExplosionSpell m_Owner;

			public InternalTarget( ExplosionSpell owner ) : base( 12, false, TargetFlags.Harmful )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
					m_Owner.Target( (Mobile)o );
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}